@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
-* Copyright (C) 1998-2003, International Business Machines Corporation and   *
-* others. All Rights Reserved.                                               *
+* Copyright (C) 1998-2012, International Business Machines Corporation and
+* others. All Rights Reserved.
 ******************************************************************************
 *
 * File schriter.cpp
@@ -13,6 +13,8 @@
 ******************************************************************************
 */
 
+#include "utypeinfo.h"  // for 'typeid' to work
+
 #include "unicode/chariter.h"
 #include "unicode/schriter.h"
 
@@ -28,31 +30,31 @@ StringCharacterIterator::StringCharacterIterator()
 }
 
 StringCharacterIterator::StringCharacterIterator(const UnicodeString& textStr)
-  : UCharCharacterIterator(textStr.fArray, textStr.length()),
+  : UCharCharacterIterator(textStr.getBuffer(), textStr.length()),
     text(textStr)
 {
     // we had set the input parameter's array, now we need to set our copy's array
-    UCharCharacterIterator::text = this->text.fArray;
+    UCharCharacterIterator::text = this->text.getBuffer();
 }
 
 StringCharacterIterator::StringCharacterIterator(const UnicodeString& textStr,
                                                  int32_t textPos)
-  : UCharCharacterIterator(textStr.fArray, textStr.length(), textPos),
+  : UCharCharacterIterator(textStr.getBuffer(), textStr.length(), textPos),
     text(textStr)
 {
     // we had set the input parameter's array, now we need to set our copy's array
-    UCharCharacterIterator::text = this->text.fArray;
+    UCharCharacterIterator::text = this->text.getBuffer();
 }
 
 StringCharacterIterator::StringCharacterIterator(const UnicodeString& textStr,
                                                  int32_t textBegin,
                                                  int32_t textEnd,
                                                  int32_t textPos)
-  : UCharCharacterIterator(textStr.fArray, textStr.length(), textBegin, textEnd, textPos),
+  : UCharCharacterIterator(textStr.getBuffer(), textStr.length(), textBegin, textEnd, textPos),
     text(textStr)
 {
     // we had set the input parameter's array, now we need to set our copy's array
-    UCharCharacterIterator::text = this->text.fArray;
+    UCharCharacterIterator::text = this->text.getBuffer();
 }
 
 StringCharacterIterator::StringCharacterIterator(const StringCharacterIterator& that)
@@ -60,7 +62,7 @@ StringCharacterIterator::StringCharacterIterator(const StringCharacterIterator&
     text(that.text)
 {
     // we had set the input parameter's array, now we need to set our copy's array
-    UCharCharacterIterator::text = this->text.fArray;
+    UCharCharacterIterator::text = this->text.getBuffer();
 }
 
 StringCharacterIterator::~StringCharacterIterator() {
@@ -71,7 +73,7 @@ StringCharacterIterator::operator=(const StringCharacterIterator& that) {
     UCharCharacterIterator::operator=(that);
     text = that.text;
     // we had set the input parameter's array, now we need to set our copy's array
-    UCharCharacterIterator::text = this->text.fArray;
+    UCharCharacterIterator::text = this->text.getBuffer();
     return *this;
 }
 
@@ -85,7 +87,7 @@ StringCharacterIterator::operator==(const ForwardCharacterIterator& that) const
     // because that checks for array pointer equality
     // while we compare UnicodeString objects
 
-    if (getDynamicClassID() != that.getDynamicClassID()) {
+    if (typeid(*this) != typeid(that)) {
         return FALSE;
     }
 
@@ -105,7 +107,7 @@ StringCharacterIterator::clone() const {
 void
 StringCharacterIterator::setText(const UnicodeString& newText) {
     text = newText;
-    UCharCharacterIterator::setText(text.fArray, text.length());
+    UCharCharacterIterator::setText(text.getBuffer(), text.length());
 }
 
 void
