@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 2010-2011, International Business Machines
+*   Copyright (C) 2010-2015, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   file name:  ucnv_ct.c
@@ -14,12 +14,13 @@
 
 #include "unicode/utypes.h"
 
-#if !UCONFIG_NO_CONVERSION && !UCONFIG_NO_LEGACY_CONVERSION
+#if !UCONFIG_NO_CONVERSION && !UCONFIG_NO_LEGACY_CONVERSION && !UCONFIG_ONLY_HTML_CONVERSION
 
 #include "unicode/ucnv.h"
 #include "unicode/uset.h"
 #include "unicode/ucnv_err.h"
 #include "unicode/ucnv_cb.h"
+#include "unicode/utf16.h"
 #include "ucnv_imp.h"
 #include "ucnv_bld.h"
 #include "ucnv_cnv.h"
@@ -27,8 +28,6 @@
 #include "cstring.h"
 #include "cmemory.h"
 
-#define LENGTHOF(array) (int32_t)(sizeof(array)/sizeof((array)[0]))
-
 typedef enum {
     INVALID = -2,
     DO_SEARCH = -1,
@@ -180,7 +179,7 @@ _CompoundTextgetName(const UConverter* cnv);
 static int32_t findNextEsc(const char *source, const char *sourceLimit) {
     int32_t length = sourceLimit - source;
     int32_t i;
-    for (i = 0; i < length; i++) {
+    for (i = 1; i < length; i++) {
         if (*(source + i) == 0x1B) {
             return i;
         }
@@ -292,7 +291,7 @@ _CompoundTextOpen(UConverter *cnv, UConverterLoadArgs *pArgs, UErrorCode *errorC
             return;
         }
 
-        myConverterData->state = 0;
+        myConverterData->state = (COMPOUND_TEXT_CONVERTERS)0;
     } else {
         *errorCode = U_MEMORY_ALLOCATION_ERROR;
     }
@@ -340,7 +339,7 @@ UConverter_fromUnicode_CompoundText_OFFSETS(UConverterFromUnicodeArgs* args, UEr
     COMPOUND_TEXT_CONVERTERS currentState, tmpState;
     uint32_t pValue;
     int32_t pValueLength = 0;
-    int32_t i, n;
+    int32_t i, n, j;
 
     UConverterDataCompoundText *myConverterData = (UConverterDataCompoundText *) cnv->extraInfo;
 
@@ -356,16 +355,16 @@ UConverter_fromUnicode_CompoundText_OFFSETS(UConverterFromUnicodeArgs* args, UEr
 
             sourceChar  = *(source++);
             /*check if the char is a First surrogate*/
-             if(UTF_IS_SURROGATE(sourceChar)) {
-                if(UTF_IS_SURROGATE_FIRST(sourceChar)) {
+             if(U16_IS_SURROGATE(sourceChar)) {
+                if(U16_IS_SURROGATE_LEAD(sourceChar)) {
 getTrail:
                     /*look ahead to find the trail surrogate*/
                     if(source < sourceLimit) {
                         /* test the following code unit */
                         UChar trail=(UChar) *source;
-                        if(UTF_IS_SECOND_SURROGATE(trail)) {
+                        if(U16_IS_TRAIL(trail)) {
                             source++;
-                            sourceChar=UTF16_GET_PAIR_VALUE(sourceChar, trail);
+                            sourceChar=U16_GET_SUPPLEMENTARY(sourceChar, trail);
                             cnv->fromUChar32=0x00;
                             /* convert this supplementary code point */
                             /* exit this condition tree */
@@ -409,8 +408,8 @@ getTrail:
                          tmpState = (COMPOUND_TEXT_CONVERTERS)i;
                          if (currentState != tmpState) {
                              currentState = tmpState;
-                             for (i = 0; escSeqCompoundText[currentState][i] != 0; i++) {
-                                 tmpTargetBuffer[tmpTargetBufferLength++] = escSeqCompoundText[currentState][i];
+                             for (j = 0; escSeqCompoundText[currentState][j] != 0; j++) {
+                                 tmpTargetBuffer[tmpTargetBufferLength++] = escSeqCompoundText[currentState][j];
                              }
                          }
                          for (n = (pValueLength - 1); n >= 0; n--) {
@@ -503,6 +502,9 @@ UConverter_toUnicode_CompoundText_OFFSETS(UConverterToUnicodeArgs *args,
                     *err = U_ZERO_ERROR;
                     break;
                 } else if (tmpState == INVALID) {
+                    if (args->converter->toULength == 0) {
+                        mySource++; /* skip over the 0x1b byte */
+                    }
                     *err = U_ILLEGAL_CHAR_FOUND;
                     break;
                 }
