@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1996-2003, International Business Machines
+*   Copyright (C) 1996-2015, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -41,6 +41,11 @@
  * \brief C++ API: Locale ID object.
  */
 
+U_NAMESPACE_BEGIN
+
+// Forward Declarations
+void U_CALLCONV locale_available_init(); /**< @internal */
+
 /**
  * A <code>Locale</code> object represents a specific geographical, political,
  * or cultural region. An operation that requires a <code>Locale</code> to perform
@@ -53,38 +58,38 @@
  * The Locale class is not suitable for subclassing.
  *
  * <P>
- * You create a <code>Locale</code> object using the constructor in
+ * You can create a <code>Locale</code> object using the constructor in
  * this class:
- * <blockquote>
+ * \htmlonly<blockquote>\endhtmlonly
  * <pre>
- * .      Locale( const   char*  language, 
- * .              const   char*  country, 
- * .              const   char*  variant);
+ *       Locale( const   char*  language,
+ *               const   char*  country,
+ *               const   char*  variant);
  * </pre>
- * </blockquote>
+ * \htmlonly</blockquote>\endhtmlonly
  * The first argument to the constructors is a valid <STRONG>ISO
  * Language Code.</STRONG> These codes are the lower-case two-letter
  * codes as defined by ISO-639.
- * You can find a full list of these codes at a number of sites, such as:
- * <BR><a href ="http://www.ics.uci.edu/pub/ietf/http/related/iso639.txt">
- * http://www.ics.uci.edu/pub/ietf/http/related/iso639.txt</a>
+ * You can find a full list of these codes at:
+ * <BR><a href ="http://www.loc.gov/standards/iso639-2/">
+ * http://www.loc.gov/standards/iso639-2/</a>
  *
  * <P>
  * The second argument to the constructors is a valid <STRONG>ISO Country
  * Code.</STRONG> These codes are the upper-case two-letter codes
  * as defined by ISO-3166.
  * You can find a full list of these codes at a number of sites, such as:
- * <BR><a href="http://www.chemie.fu-berlin.de/diverse/doc/ISO_3166.html">
- * http://www.chemie.fu-berlin.de/diverse/doc/ISO_3166.html</a>
+ * <BR><a href="http://www.iso.org/iso/en/prods-services/iso3166ma/index.html">
+ * http://www.iso.org/iso/en/prods-services/iso3166ma/index.html</a>
  *
  * <P>
  * The third constructor requires a third argument--the <STRONG>Variant.</STRONG>
  * The Variant codes are vendor and browser-specific.
- * For example, use WIN for Windows, MAC for Macintosh, and POSIX for POSIX.
+ * For example, use REVISED for a langauge's revised script orthography, and POSIX for POSIX.
  * Where there are two variants, separate them with an underscore, and
  * put the most important one first. For
  * example, a Traditional Spanish collation might be referenced, with
- * "ES", "ES", "Traditional_WIN".
+ * "ES", "ES", "Traditional_POSIX".
  *
  * <P>
  * Because a <code>Locale</code> object is just an identifier for a region,
@@ -103,11 +108,11 @@
  * that you can use to create <code>Locale</code> objects for commonly used
  * locales. For example, the following refers to a <code>Locale</code> object
  * for the United States:
- * <blockquote>
+ * \htmlonly<blockquote>\endhtmlonly
  * <pre>
  *       Locale::getUS()
  * </pre>
- * </blockquote>
+ * \htmlonly</blockquote>\endhtmlonly
  *
  * <P>
  * Once you've created a <code>Locale</code> you can query it for information about
@@ -123,33 +128,33 @@
  * a language appropriate to that locale.
  *
  * <P>
- * The TIFC provides a number of classes that perform locale-sensitive
+ * ICU provides a number of classes that perform locale-sensitive
  * operations. For example, the <code>NumberFormat</code> class formats
  * numbers, currency, or percentages in a locale-sensitive manner. Classes
  * such as <code>NumberFormat</code> have a number of convenience methods
  * for creating a default object of that type. For example, the
  * <code>NumberFormat</code> class provides these three convenience methods
  * for creating a default <code>NumberFormat</code> object:
- * <blockquote>
+ * \htmlonly<blockquote>\endhtmlonly
  * <pre>
  *     UErrorCode success = U_ZERO_ERROR;
  *     Locale myLocale;
  *     NumberFormat *nf;
- * 
+ *
  *     nf = NumberFormat::createInstance( success );          delete nf;
  *     nf = NumberFormat::createCurrencyInstance( success );  delete nf;
  *     nf = NumberFormat::createPercentInstance( success );   delete nf;
  * </pre>
- * </blockquote>
+ * \htmlonly</blockquote>\endhtmlonly
  * Each of these methods has two variants; one with an explicit locale
  * and one without; the latter using the default locale.
- * <blockquote>
+ * \htmlonly<blockquote>\endhtmlonly
  * <pre>
  *     nf = NumberFormat::createInstance( myLocale, success );          delete nf;
  *     nf = NumberFormat::createCurrencyInstance( myLocale, success );  delete nf;
  *     nf = NumberFormat::createPercentInstance( myLocale, success );   delete nf;
  * </pre>
- * </blockquote>
+ * \htmlonly</blockquote>\endhtmlonly
  * A <code>Locale</code> is the mechanism for identifying the kind of object
  * (<code>NumberFormat</code>) that you would like to get. The locale is
  * <STRONG>just</STRONG> a mechanism for identifying objects,
@@ -163,7 +168,7 @@
  * For example, you can create a menu of all the collation objects
  * suitable for a given language. Such classes implement these
  * three class methods:
- * <blockquote>
+ * \htmlonly<blockquote>\endhtmlonly
  * <pre>
  *       static Locale* getAvailableLocales(int32_t& numLocales)
  *       static UnicodeString& getDisplayName(const Locale&  objectLocale,
@@ -172,57 +177,58 @@
  *       static UnicodeString& getDisplayName(const Locale&  objectLocale,
  *                                            UnicodeString& displayName)
  * </pre>
- * </blockquote>
+ * \htmlonly</blockquote>\endhtmlonly
  *
  * @stable ICU 2.0
  * @see ResourceBundle
  */
-U_NAMESPACE_BEGIN
 class U_COMMON_API Locale : public UObject {
 public:
+    /** Useful constant for the Root locale. @stable ICU 4.4 */
+    static const Locale &U_EXPORT2 getRoot(void);
     /** Useful constant for this language. @stable ICU 2.0 */
-    static const Locale &getEnglish(void);
+    static const Locale &U_EXPORT2 getEnglish(void);
     /** Useful constant for this language. @stable ICU 2.0 */
-    static const Locale &getFrench(void);
+    static const Locale &U_EXPORT2 getFrench(void);
     /** Useful constant for this language. @stable ICU 2.0 */
-    static const Locale &getGerman(void);
+    static const Locale &U_EXPORT2 getGerman(void);
     /** Useful constant for this language. @stable ICU 2.0 */
-    static const Locale &getItalian(void);
+    static const Locale &U_EXPORT2 getItalian(void);
     /** Useful constant for this language. @stable ICU 2.0 */
-    static const Locale &getJapanese(void);
+    static const Locale &U_EXPORT2 getJapanese(void);
     /** Useful constant for this language. @stable ICU 2.0 */
-    static const Locale &getKorean(void);
+    static const Locale &U_EXPORT2 getKorean(void);
     /** Useful constant for this language. @stable ICU 2.0 */
-    static const Locale &getChinese(void);
+    static const Locale &U_EXPORT2 getChinese(void);
     /** Useful constant for this language. @stable ICU 2.0 */
-    static const Locale &getSimplifiedChinese(void);
+    static const Locale &U_EXPORT2 getSimplifiedChinese(void);
     /** Useful constant for this language. @stable ICU 2.0 */
-    static const Locale &getTraditionalChinese(void);
+    static const Locale &U_EXPORT2 getTraditionalChinese(void);
 
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getFrance(void);
+    static const Locale &U_EXPORT2 getFrance(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getGermany(void);
+    static const Locale &U_EXPORT2 getGermany(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getItaly(void);
+    static const Locale &U_EXPORT2 getItaly(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getJapan(void);
+    static const Locale &U_EXPORT2 getJapan(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getKorea(void);
+    static const Locale &U_EXPORT2 getKorea(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getChina(void);
+    static const Locale &U_EXPORT2 getChina(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getPRC(void);
+    static const Locale &U_EXPORT2 getPRC(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getTaiwan(void);
+    static const Locale &U_EXPORT2 getTaiwan(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getUK(void);
+    static const Locale &U_EXPORT2 getUK(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getUS(void);
+    static const Locale &U_EXPORT2 getUS(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getCanada(void);
+    static const Locale &U_EXPORT2 getCanada(void);
     /** Useful constant for this country/region. @stable ICU 2.0 */
-    static const Locale &getCanadaFrench(void);
+    static const Locale &U_EXPORT2 getCanadaFrench(void);
 
 
     /**
@@ -232,7 +238,7 @@ public:
      * @see uloc_getDefault
      * @stable ICU 2.0
      */
-    Locale(); 
+    Locale();
 
     /**
      * Construct a locale from language, country, variant.
@@ -245,9 +251,9 @@ public:
      *  This parameter can be NULL; if so,
      *  the locale is initialized to match the current default locale.
      *  (This is the same as using the default constructor.)
-     *  Please note: The Java Locale class does NOT accept the form 
+     *  Please note: The Java Locale class does NOT accept the form
      *  'new Locale("en_US")' but only 'new Locale("en","US")'
-     * 
+     *
      * @param country  Uppercase two-letter ISO-3166 code. (optional)
      * @param variant  Uppercase vendor and browser specific code. See class
      *                 description. (optional)
@@ -259,7 +265,7 @@ public:
      * @stable ICU 2.0
      */
     Locale( const   char * language,
-            const   char * country  = 0, 
+            const   char * country  = 0,
             const   char * variant  = 0,
             const   char * keywordsAndValues = 0);
 
@@ -276,7 +282,7 @@ public:
      * Destructor
      * @stable ICU 2.0
      */
-    ~Locale() ;
+    virtual ~Locale() ;
 
     /**
      * Replaces the entire contents of *this with the specified value.
@@ -315,10 +321,11 @@ public:
      * @return a clone of this object
      *
      * @see getDynamicClassID
-     * @draft ICU 2.8
+     * @stable ICU 2.8
      */
     Locale *clone() const;
 
+#ifndef U_HIDE_SYSTEM_API
     /**
      * Common methods of getting the current default Locale. Used for the
      * presentation: menus, dialogs, etc. Generally set once when your applet or
@@ -334,7 +341,7 @@ public:
      * @system
      * @stable ICU 2.0
      */
-    static  const Locale& getDefault(void);
+    static const Locale& U_EXPORT2 getDefault(void);
 
     /**
      * Sets the default. Normally set once at the beginning of a process,
@@ -348,22 +355,31 @@ public:
      * @system
      * @stable ICU 2.0
      */
-    static  void    setDefault(const    Locale&     newLocale,
-                               UErrorCode&  success);
+    static void U_EXPORT2 setDefault(const Locale& newLocale,
+                                     UErrorCode&   success);
+#endif  /* U_HIDE_SYSTEM_API */
 
-    
     /**
-     * Creates a locale which has had minimal canonicalization 
-     * as per uloc_getName(). 
+     * Creates a locale which has had minimal canonicalization
+     * as per uloc_getName().
      * @param name The name to create from.  If name is null,
      *  the default Locale is used.
      * @return new locale object
      * @stable ICU 2.0
      * @see uloc_getName
      */
-    static Locale createFromName(const char *name);
+    static Locale U_EXPORT2 createFromName(const char *name);
+
+    /**
+     * Creates a locale from the given string after canonicalizing
+     * the string by calling uloc_canonicalize().
+     * @param name the locale ID to create from.  Must not be NULL.
+     * @return a new locale object corresponding to the given name
+     * @stable ICU 3.0
+     * @see uloc_canonicalize
+     */
+    static Locale U_EXPORT2 createCanonical(const char* name);
 
-    
     /**
      * Returns the locale's ISO-639 language code.
      * @return      An alias to the code
@@ -376,7 +392,7 @@ public:
      * @return      An alias to the code
      * @see uscript_getShortName
      * @see uscript_getCode
-     * @draft ICU 2.8
+     * @stable ICU 2.8
      */
     inline const char *  getScript( ) const;
 
@@ -405,47 +421,65 @@ public:
     inline const char * getName() const;
 
     /**
-     * Returns the programmatic name of the entire locale as getName would return,
+     * Returns the programmatic name of the entire locale as getName() would return,
      * but without keywords.
      * @return      A pointer to "name".
      * @see getName
-     * @draft ICU 2.8
+     * @stable ICU 2.8
      */
     const char * getBaseName() const;
 
 
     /**
-     * Gets the list of keywords for the specified locale. 
+     * Gets the list of keywords for the specified locale.
      *
-     * @return pointer to StringEnumeration class. Client must dispose of it by calling delete.
-     * @param status Returns any error information while performing this operation.
-     * @draft ICU 2.8
+     * @param status the status code
+     * @return pointer to StringEnumeration class, or NULL if there are no keywords. 
+     * Client must dispose of it by calling delete.
+     * @stable ICU 2.8
      */
     StringEnumeration * createKeywords(UErrorCode &status) const;
 
     /**
-     * Get the value for a keyword. 
-     * 
-     * @param keywordName name of the keyword for which we want the value. Case insensitive.
-     * @param status Returns any error information while performing this operation.
-     * @return pointer to the keyword value owned by the Locale object or NULL if there is 
-     *         no such a keyword.
+     * Gets the value for a keyword.
      *
-     * @draft ICU 2.8
+     * @param keywordName name of the keyword for which we want the value. Case insensitive.
+     * @param buffer The buffer to receive the keyword value.
+     * @param bufferCapacity The capacity of receiving buffer
+     * @param status Returns any error information while performing this operation.
+     * @return the length of the keyword value
+     *
+     * @stable ICU 2.8
      */
-     int32_t getKeywordValue(const char* keywordName, char *buffer, int32_t bufLen, UErrorCode &status) const;
+    int32_t getKeywordValue(const char* keywordName, char *buffer, int32_t bufferCapacity, UErrorCode &status) const;
+
+    /**
+     * Sets or removes the value for a keyword.
+     *
+     * For removing all keywords, use getBaseName(),
+     * and construct a new Locale if it differs from getName().
+     *
+     * @param keywordName name of the keyword to be set. Case insensitive.
+     * @param keywordValue value of the keyword to be set. If 0-length or
+     *  NULL, will result in the keyword being removed. No error is given if
+     *  that keyword does not exist.
+     * @param status Returns any error information while performing this operation.
+     *
+     * @stable ICU 49
+     */
+    void setKeywordValue(const char* keywordName, const char* keywordValue, UErrorCode &status);
 
     /**
      * returns the locale's three-letter language code, as specified
-     * in ISO draft standard ISO-639-2..
-     * @return      An alias to the code, or NULL
+     * in ISO draft standard ISO-639-2.
+     * @return      An alias to the code, or an empty string
      * @stable ICU 2.0
      */
     const char * getISO3Language() const;
 
     /**
      * Fills in "name" with the locale's three-letter ISO-3166 country code.
-     * @return      An alias to the code, or NULL
+     * @return      An alias to the code, or an empty string
      * @stable ICU 2.0
      */
     const char * getISO3Country() const;
@@ -459,6 +493,23 @@ public:
      */
     uint32_t        getLCID(void) const;
 
+#ifndef U_HIDE_DRAFT_API
+    /**
+     * Returns whether this locale's script is written right-to-left.
+     * If there is no script subtag, then the likely script is used, see uloc_addLikelySubtags().
+     * If no likely script is known, then FALSE is returned.
+     *
+     * A script is right-to-left according to the CLDR script metadata
+     * which corresponds to whether the script's letters have Bidi_Class=R or AL.
+     *
+     * Returns TRUE for "ar" and "en-Hebr", FALSE for "zh" and "fa-Cyrl".
+     *
+     * @return TRUE if the locale's script is written right-to-left
+     * @draft ICU 54
+     */
+    UBool isRightToLeft() const;
+#endif  /* U_HIDE_DRAFT_API */
+
     /**
      * Fills in "dispLang" with the name of this locale's language in a format suitable for
      * user display in the default locale.  For example, if the locale's language code is
@@ -493,7 +544,7 @@ public:
      * dispScript to "Latin".
      * @param dispScript    Receives the scripts's display name.
      * @return              A reference to "dispScript".
-     * @draft ICU 2.8
+     * @stable ICU 2.8
      */
     UnicodeString&  getDisplayScript(          UnicodeString& dispScript) const;
 
@@ -509,7 +560,7 @@ public:
      *                      "".
      * @param dispScript    Receives the scripts's display name.
      * @return              A reference to "dispScript".
-     * @draft ICU 2.8
+     * @stable ICU 2.8
      */
     UnicodeString&  getDisplayScript(  const   Locale&         displayLocale,
                                                UnicodeString&  dispScript) const;
@@ -529,10 +580,10 @@ public:
      * Fills in "dispCountry" with the name of this locale's country in a format suitable
      * for user display in the locale specified by "displayLocale".  For example, if the locale's
      * country code is "US" and displayLocale's language code is "fr", this function would set
-     * dispCountry to "Etats-Unis".
+     * dispCountry to "&Eacute;tats-Unis".
      * @param displayLocale      Specifies the locale to be used to display the name.  In other
      *                      words, if the locale's country code is "US", passing
-     *                      Locale::getFrench() for displayLocale would result in "tats-Unis", while
+     *                      Locale::getFrench() for displayLocale would result in "&Eacute;tats-Unis", while
      *                      passing Locale::getGerman() for displayLocale would result in
      *                      "Vereinigte Staaten".
      * @param dispCountry   Receives the country's display name.
@@ -563,7 +614,7 @@ public:
                                                 UnicodeString&  dispVar) const;
 
     /**
-     * Fills in "name" with the name of this locale in a format suitable for user display 
+     * Fills in "name" with the name of this locale in a format suitable for user display
      * in the default locale.  This function uses getDisplayLanguage(), getDisplayCountry(),
      * and getDisplayVariant() to do its work, and outputs the display name in the format
      * "language (country[,variant])".  For example, if the default locale is en_US, then
@@ -576,12 +627,12 @@ public:
     UnicodeString&  getDisplayName(         UnicodeString&  name) const;
 
     /**
-     * Fills in "name" with the name of this locale in a format suitable for user display 
+     * Fills in "name" with the name of this locale in a format suitable for user display
      * in the locale specfied by "displayLocale".  This function uses getDisplayLanguage(),
      * getDisplayCountry(), and getDisplayVariant() to do its work, and outputs the display
      * name in the format "language (country[,variant])".  For example, if displayLocale is
-     * fr_FR, then en_US's display name would be "Anglais (tats-Unis)", and no_NO_NY's
-     * display name would be "norvgien (Norvge,NY)".
+     * fr_FR, then en_US's display name would be "Anglais (&Eacute;tats-Unis)", and no_NO_NY's
+     * display name would be "norv&eacute;gien (Norv&egrave;ge,NY)".
      * @param displayLocale  Specifies the locale to be used to display the name.
      * @param name      Receives the locale's display name.
      * @return          A reference to "name".
@@ -596,12 +647,12 @@ public:
      */
     int32_t         hashCode(void) const;
 
-    /** 
-     * Sets the locale to bogus 
-     * A bogus locale represents a non-existing locale associated 
-     * with services that can be instantiated from non-locale data 
-     * in addition to locale (for example, collation can be 
-     * instantiated from a locale and from a rule set). 
+    /**
+     * Sets the locale to bogus
+     * A bogus locale represents a non-existing locale associated
+     * with services that can be instantiated from non-locale data
+     * in addition to locale (for example, collation can be
+     * instantiated from a locale and from a rule set).
      * @stable ICU 2.1
      */
     void setToBogus();
@@ -621,17 +672,17 @@ public:
      *              get ownership of this list, and must NOT delete it.
      * @stable ICU 2.0
      */
-    static  const   Locale*     getAvailableLocales(int32_t& count);
+    static const Locale* U_EXPORT2 getAvailableLocales(int32_t& count);
 
     /**
-     * Gets a list of all available 2-letter country codes defined in ISO 639.  This is a
+     * Gets a list of all available 2-letter country codes defined in ISO 3166.  This is a
      * pointer to an array of pointers to arrays of char.  All of these pointers are
      * owned by ICU-- do not delete them, and do not write through them.  The array is
      * terminated with a null pointer.
      * @return a list of all available country codes
      * @stable ICU 2.0
      */
-    static const char* const* getISOCountries();
+    static const char* const* U_EXPORT2 getISOCountries();
 
     /**
      * Gets a list of all available language codes defined in ISO 639.  This is a pointer
@@ -641,14 +692,14 @@ public:
      * @return a list of all available language codes
      * @stable ICU 2.0
      */
-    static const char* const*  getISOLanguages();
+    static const char* const* U_EXPORT2 getISOLanguages();
 
     /**
      * ICU "poor man's RTTI", returns a UClassID for this class.
      *
      * @stable ICU 2.2
      */
-    static UClassID getStaticClassID();
+    static UClassID U_EXPORT2 getStaticClassID();
 
     /**
      * ICU "poor man's RTTI", returns a UClassID for the actual class.
@@ -658,11 +709,13 @@ public:
     virtual UClassID getDynamicClassID() const;
 
 protected: /* only protected for testing purposes. DO NOT USE. */
+#ifndef U_HIDE_INTERNAL_API
     /**
      * Set this from a single POSIX style locale string.
      * @internal
      */
     void setFromPOSIXID(const char *posixID);
+#endif  /* U_HIDE_INTERNAL_API */
 
 private:
     /**
@@ -670,8 +723,9 @@ private:
      * Was deprecated - used in implementation - moved internal
      *
      * @param cLocaleID The new locale name.
+     * @param canonicalize whether to call uloc_canonicalize on cLocaleID
      */
-    Locale& init(const char* cLocaleID);
+    Locale& init(const char* cLocaleID, UBool canonicalize);
 
     /*
      * Internal constructor to allow construction of a locale object with
@@ -696,7 +750,7 @@ private:
     char fullNameBuffer[ULOC_FULLNAME_CAPACITY];
     // name without keywords
     char* baseName;
-    char baseNameBuffer[ULOC_FULLNAME_CAPACITY];
+    void initBaseName(UErrorCode& status);
 
     UBool fIsBogus;
 
@@ -706,7 +760,12 @@ private:
      * A friend to allow the default locale to be set by either the C or C++ API.
      * @internal
      */
-    friend void locale_set_default_internal(const char *);
+    friend Locale *locale_set_default_internal(const char *, UErrorCode& status);
+
+    /**
+     * @internal
+     */
+    friend void U_CALLCONV locale_available_init();
 };
 
 inline UBool
@@ -736,16 +795,16 @@ Locale::getScript() const
 inline const char *
 Locale::getVariant() const
 {
-    return &fullName[variantBegin];
+    return &baseName[variantBegin];
 }
 
-inline const char * 
+inline const char *
 Locale::getName() const
 {
     return fullName;
 }
 
-inline UBool 
+inline UBool
 Locale::isBogus(void) const {
     return fIsBogus;
 }
@@ -753,4 +812,3 @@ Locale::isBogus(void) const {
 U_NAMESPACE_END
 
 #endif
-
