@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-*   Copyright (C) 2003, International Business Machines
+*   Copyright (C) 2003-2014, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *******************************************************************************
 *   file name:  utrace.c
@@ -23,7 +23,7 @@ static UTraceData      *pTraceDataFunc  = NULL;
 static const void      *gTraceContext   = NULL;
 
 U_EXPORT int32_t
-utrace_level;
+utrace_level = UTRACE_ERROR;
 
 U_CAPI void U_EXPORT2
 utrace_entry(int32_t fnNumber) {
@@ -132,16 +132,15 @@ static void outputHexBytes(int64_t val, int32_t charsToOutput,
 
 /* Output a pointer value in hex.  Work with any size of pointer   */
 static void outputPtrBytes(void *val, char *outBuf, int32_t *outIx, int32_t capacity) {
-    static const int16_t endianTestVal = (int16_t)0xabcd;
     int32_t  i;
     int32_t  incVal = 1;              /* +1 for big endian, -1 for little endian          */
     char     *p     = (char *)&val;   /* point to current byte to output in the ptr val  */
 
-    if (*(uint8_t *)&endianTestVal == (uint8_t)0xcd) {
-        /* Little Endian.  Move p to most significant end of the value      */
-        incVal = -1;
-        p += sizeof(void *) - 1;
-    }
+#if !U_IS_BIG_ENDIAN
+    /* Little Endian.  Move p to most significant end of the value      */
+    incVal = -1;
+    p += sizeof(void *) - 1;
+#endif
 
     /* Loop through the bytes of the ptr as it sits in memory, from 
      * most significant to least significant end                    */
@@ -231,7 +230,7 @@ utrace_vformat(char *outBuf, int32_t capacity, int32_t indent, const char *fmt,
             /* UChar * string, with length, len==-1 for null terminated. */
             ptrArg = va_arg(args, void *);             /* Ptr    */
             intArg =(int32_t)va_arg(args, int32_t);    /* Length */
-            outputUString((const unsigned short *)ptrArg, intArg, outBuf, &outIx, capacity, indent);
+            outputUString((const UChar *)ptrArg, intArg, outBuf, &outIx, capacity, indent);
             break;
 
         case 'b':
@@ -339,7 +338,7 @@ utrace_vformat(char *outBuf, int32_t capacity, int32_t indent, const char *fmt,
 
                         case 'S':
                             charsToOutput = 0;
-                            outputUString((const unsigned short *)*ptrPtr, -1, outBuf, &outIx, capacity, indent);
+                            outputUString((const UChar *)*ptrPtr, -1, outBuf, &outIx, capacity, indent);
                             outputChar('\n', outBuf, &outIx, capacity, indent);
                             longArg = *ptrPtr==NULL? 0: 1;   /* for test for null term. array. */
                             ptrPtr++;
@@ -468,6 +467,8 @@ trCollNames[] = {
     "ucol_getLocale",
     "ucol_nextSortKeyPart",
     "ucol_strcollIter",
+    "ucol_openFromShortString",
+    "ucol_strcollUTF8",
     NULL
 };
 
