@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-*   Copyright (C) 1997-2003, International Business Machines
+*   Copyright (C) 1997-2011,2014-2015 International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *******************************************************************************
 *   Date        Name        Description
@@ -15,9 +15,11 @@
 
 #if !UCONFIG_NO_TRANSLITERATION
 
+#include "unicode/localpointer.h"
 #include "unicode/urep.h"
 #include "unicode/parseerr.h"
 #include "unicode/uenum.h"
+#include "unicode/uset.h"
 
 /********************************************************************
  * General Notes
@@ -76,16 +78,16 @@ typedef void* UTransliterator;
 typedef enum UTransDirection {
     
     /**
-     * UTRANS_FORWARD means from <source> to <target> for a
-     * transliterator with ID <source>-<target>.  For a transliterator
+     * UTRANS_FORWARD means from &lt;source&gt; to &lt;target&gt; for a
+     * transliterator with ID &lt;source&gt;-&lt;target&gt;.  For a transliterator
      * opened using a rule, it means forward direction rules, e.g.,
      * "A > B".
      */
     UTRANS_FORWARD,
 
     /**
-     * UTRANS_REVERSE means from <target> to <source> for a
-     * transliterator with ID <source>-<target>.  For a transliterator
+     * UTRANS_REVERSE means from &lt;target&gt; to &lt;source&gt; for a
+     * transliterator with ID &lt;source&gt;-&lt;target&gt;.  For a transliterator
      * opened using a rule, it means reverse direction rules, e.g.,
      * "A < B".
      */
@@ -123,7 +125,7 @@ typedef struct UTransPosition {
      * anything before this index.  INPUT/OUTPUT parameter: This parameter
      * is updated by a transliteration operation to reflect the maximum
      * amount of antecontext needed by a transliterator.
-     * @draft ICU 2.4
+     * @stable ICU 2.4
      */
     int32_t contextStart;
     
@@ -133,7 +135,7 @@ typedef struct UTransPosition {
      * anything at or after this index.  INPUT/OUTPUT parameter: This
      * parameter is updated to reflect changes in the length of the
      * text, but points to the same logical position in the text.
-     * @draft ICU 2.4
+     * @stable ICU 2.4
      */
     int32_t contextLimit;
     
@@ -142,7 +144,7 @@ typedef struct UTransPosition {
      * INPUT/OUTPUT parameter: This parameter is advanced past
      * characters that have already been transliterated by a
      * transliteration operation.
-     * @draft ICU 2.4
+     * @stable ICU 2.4
      */
     int32_t start;
     
@@ -151,7 +153,7 @@ typedef struct UTransPosition {
      * INPUT/OUTPUT parameter: This parameter is updated to reflect
      * changes in the length of the text, but points to the same
      * logical position in the text.
-     * @draft ICU 2.4
+     * @stable ICU 2.4
      */
     int32_t limit;
 
@@ -182,9 +184,9 @@ typedef struct UTransPosition {
  * @param pErrorCode a pointer to the UErrorCode
  * @return a transliterator pointer that may be passed to other
  *         utrans_xxx() functions, or NULL if the open call fails.
- * @draft ICU 2.8
+ * @stable ICU 2.8
  */
-U_CAPI UTransliterator* U_EXPORT2
+U_STABLE UTransliterator* U_EXPORT2
 utrans_openU(const UChar *id,
              int32_t idLength,
              UTransDirection dir,
@@ -208,7 +210,7 @@ utrans_openU(const UChar *id,
  * inverse of trans, or NULL if the open call fails.
  * @stable ICU 2.0
  */
-U_CAPI UTransliterator* U_EXPORT2 
+U_STABLE UTransliterator* U_EXPORT2 
 utrans_openInverse(const UTransliterator* trans,
                    UErrorCode* status);
 
@@ -222,7 +224,7 @@ utrans_openInverse(const UTransliterator* trans,
  * utrans_xxx() functions, or NULL if the clone call fails.
  * @stable ICU 2.0
  */
-U_CAPI UTransliterator* U_EXPORT2 
+U_STABLE UTransliterator* U_EXPORT2 
 utrans_clone(const UTransliterator* trans,
              UErrorCode* status);
 
@@ -232,9 +234,28 @@ utrans_clone(const UTransliterator* trans,
  * @param trans the transliterator to be closed.
  * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_close(UTransliterator* trans);
 
+#if U_SHOW_CPLUSPLUS_API
+
+U_NAMESPACE_BEGIN
+
+/**
+ * \class LocalUTransliteratorPointer
+ * "Smart pointer" class, closes a UTransliterator via utrans_close().
+ * For most methods see the LocalPointerBase base class.
+ *
+ * @see LocalPointerBase
+ * @see LocalPointer
+ * @stable ICU 4.4
+ */
+U_DEFINE_LOCAL_OPEN_POINTER(LocalUTransliteratorPointer, UTransliterator, utrans_close);
+
+U_NAMESPACE_END
+
+#endif
+
 /**
  * Return the programmatic identifier for this transliterator.
  * If this identifier is passed to utrans_openU(), it will open
@@ -244,11 +265,12 @@ utrans_close(UTransliterator* trans);
  * @param trans the transliterator to return the ID of.
  * @param resultLength pointer to an output variable receiving the length
  *        of the ID string; can be NULL
- * @return the NUL-terminated ID string
+ * @return the NUL-terminated ID string. This pointer remains
+ * valid until utrans_close() is called on this transliterator.
  *
- * @draft ICU 2.8
+ * @stable ICU 2.8
  */
-U_CAPI const UChar * U_EXPORT2
+U_STABLE const UChar * U_EXPORT2
 utrans_getUnicodeID(const UTransliterator *trans,
                     int32_t *resultLength);
 
@@ -266,7 +288,7 @@ utrans_getUnicodeID(const UTransliterator *trans,
  * @param status a pointer to the UErrorCode
  * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_register(UTransliterator* adoptedTrans,
                 UErrorCode* status);
 
@@ -277,9 +299,9 @@ utrans_register(UTransliterator* adoptedTrans,
  *
  * @param id an ID to unregister
  * @param idLength the length of id, or -1 if id is zero-terminated
- * @draft ICU 2.8
+ * @stable ICU 2.8
  */
-U_CAPI void U_EXPORT2
+U_STABLE void U_EXPORT2
 utrans_unregisterID(const UChar* id, int32_t idLength);
 
 /**
@@ -300,7 +322,7 @@ utrans_unregisterID(const UChar* id, int32_t idLength);
  * @see UnicodeSet
  * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_setFilter(UTransliterator* trans,
                  const UChar* filterPattern,
                  int32_t filterPatternLen,
@@ -313,7 +335,7 @@ utrans_setFilter(UTransliterator* trans,
  * @return the number of system transliterators.
  * @stable ICU 2.0
  */
-U_CAPI int32_t U_EXPORT2 
+U_STABLE int32_t U_EXPORT2 
 utrans_countAvailableIDs(void);
 
 /**
@@ -323,9 +345,9 @@ utrans_countAvailableIDs(void);
  * @return UEnumeration for the available transliterators.
  *         Close with uenum_close().
  *
- * @draft ICU 2.8
+ * @stable ICU 2.8
  */
-U_CAPI UEnumeration * U_EXPORT2
+U_STABLE UEnumeration * U_EXPORT2
 utrans_openIDs(UErrorCode *pErrorCode);
 
 /********************************************************************
@@ -355,7 +377,7 @@ utrans_openIDs(UErrorCode *pErrorCode);
  * @param status a pointer to the UErrorCode
  * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_trans(const UTransliterator* trans,
              UReplaceable* rep,
              UReplaceableCallbacks* repFunc,
@@ -406,7 +428,7 @@ utrans_trans(const UTransliterator* trans,
  * @param status a pointer to the UErrorCode
  * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_transIncremental(const UTransliterator* trans,
                         UReplaceable* rep,
                         UReplaceableCallbacks* repFunc,
@@ -444,7 +466,7 @@ utrans_transIncremental(const UTransliterator* trans,
  * @param status a pointer to the UErrorCode
  * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_transUChars(const UTransliterator* trans,
                    UChar* text,
                    int32_t* textLength,
@@ -479,7 +501,7 @@ utrans_transUChars(const UTransliterator* trans,
  * @see utrans_transIncremental
  * @stable ICU 2.0
  */
-U_CAPI void U_EXPORT2 
+U_STABLE void U_EXPORT2 
 utrans_transIncrementalUChars(const UTransliterator* trans,
                               UChar* text,
                               int32_t* textLength,
@@ -487,8 +509,58 @@ utrans_transIncrementalUChars(const UTransliterator* trans,
                               UTransPosition* pos,
                               UErrorCode* status);
 
+/**
+ * Create a rule string that can be passed to utrans_openU to recreate this
+ * transliterator.
+ *
+ * @param trans     The transliterator
+ * @param escapeUnprintable if TRUE then convert unprintable characters to their
+ *                  hex escape representations, \\uxxxx or \\Uxxxxxxxx.
+ *                  Unprintable characters are those other than
+ *                  U+000A, U+0020..U+007E.
+ * @param result    A pointer to a buffer to receive the rules.
+ * @param resultLength The maximum size of result.
+ * @param status    A pointer to the UErrorCode. In case of error status, the
+ *                  contents of result are undefined.
+ * @return int32_t   The length of the rule string (may be greater than resultLength,
+ *                  in which case an error is returned).
+ * @stable ICU 53
+ */
+U_STABLE int32_t U_EXPORT2
+utrans_toRules(     const UTransliterator* trans,
+                    UBool escapeUnprintable,
+                    UChar* result, int32_t resultLength,
+                    UErrorCode* status);
+
+/**
+ * Returns the set of all characters that may be modified in the input text by
+ * this UTransliterator, optionally ignoring the transliterator's current filter.
+ * @param trans     The transliterator.
+ * @param ignoreFilter If FALSE, the returned set incorporates the
+ *                  UTransliterator's current filter; if the filter is changed,
+ *                  the return value of this function will change. If TRUE, the
+ *                  returned set ignores the effect of the UTransliterator's
+ *                  current filter.
+ * @param fillIn    Pointer to a USet object to receive the modifiable characters
+ *                  set. Previous contents of fillIn are lost. <em>If fillIn is
+ *                  NULL, then a new USet is created and returned. The caller
+ *                  owns the result and must dispose of it by calling uset_close.</em>
+ * @param status    A pointer to the UErrorCode.
+ * @return USet*    Either fillIn, or if fillIn is NULL, a pointer to a
+ *                  newly-allocated USet that the user must close. In case of
+ *                  error, NULL is returned.
+ * @stable ICU 53
+ */
+U_STABLE USet* U_EXPORT2
+utrans_getSourceSet(const UTransliterator* trans,
+                    UBool ignoreFilter,
+                    USet* fillIn,
+                    UErrorCode* status);
+
 /* deprecated API ----------------------------------------------------------- */
 
+#ifndef U_HIDE_DEPRECATED_API
+
 /* see utrans.h documentation for why these functions are deprecated */
 
 /**
@@ -514,7 +586,7 @@ utrans_transIncrementalUChars(const UTransliterator* trans,
  * utrans_xxx() functions, or NULL if the open call fails.
  * @deprecated ICU 2.8 Use utrans_openU() instead, see utrans.h
  */
-U_CAPI UTransliterator* U_EXPORT2 
+U_DEPRECATED UTransliterator* U_EXPORT2 
 utrans_open(const char* id,
             UTransDirection dir,
             const UChar* rules,         /* may be Null */
@@ -537,7 +609,7 @@ utrans_open(const char* id,
  * zero-termination.  This may be greater than bufCapacity.
  * @deprecated ICU 2.8 Use utrans_getUnicodeID() instead, see utrans.h
  */
-U_CAPI int32_t U_EXPORT2 
+U_DEPRECATED int32_t U_EXPORT2 
 utrans_getID(const UTransliterator* trans,
              char* buf,
              int32_t bufCapacity);
@@ -551,7 +623,7 @@ utrans_getID(const UTransliterator* trans,
  * @param id a zero-terminated ID
  * @deprecated ICU 2.8 Use utrans_unregisterID() instead, see utrans.h
  */
-U_CAPI void U_EXPORT2 
+U_DEPRECATED void U_EXPORT2 
 utrans_unregister(const char* id);
 
 /**
@@ -572,11 +644,13 @@ utrans_unregister(const char* id);
  * zero-termination.  This may be greater than bufCapacity.
  * @deprecated ICU 2.8 Use utrans_openIDs() instead, see utrans.h
  */
-U_CAPI int32_t U_EXPORT2 
+U_DEPRECATED int32_t U_EXPORT2 
 utrans_getAvailableID(int32_t index,
                       char* buf,
                       int32_t bufCapacity);
 
+#endif  /* U_HIDE_DEPRECATED_API */
+
 #endif /* #if !UCONFIG_NO_TRANSLITERATION */
 
 #endif
