@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1996-2003, International Business Machines Corporation and
+ * Copyright (c) 1996-2012, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -28,48 +28,61 @@ void addUTF16Test(TestNode** root);
 void addUTF8Test(TestNode** root);
 void addUTransTest(TestNode** root);
 void addPUtilTest(TestNode** root);
-void addCompactArrayTest(TestNode** root);
-void addTestDeprecatedAPI(TestNode** root);
 void addUCharTransformTest(TestNode** root);
 void addUSetTest(TestNode** root);
 void addUStringPrepTest(TestNode** root);
 void addIDNATest(TestNode** root);
 void addHeapMutexTest(TestNode **root);
 void addUTraceTest(TestNode** root);
-
+void addURegexTest(TestNode** root);
+void addUTextTest(TestNode** root);
+void addUCsdetTest(TestNode** root);
+void addCnvSelTest(TestNode** root);
+void addUSpoofTest(TestNode** root);
+#if !UCONFIG_NO_FORMATTING
+void addGendInfoForTest(TestNode** root);
+#endif
 
 void addAllTests(TestNode** root)
 {
+    addCnvSelTest(root);
     addUDataTest(root);
-    addPUtilTest(root);
+    addHeapMutexTest(root);
     addUTF16Test(root);
     addUTF8Test(root);
     addUtility(root);
+    addUTraceTest(root);
+    addUTextTest(root);
     addConvert(root);
     addUCharTransformTest(root);
     addStandardNamesTest(root);
-    addCompactArrayTest(root);
-#if !UCONFIG_NO_FORMATTING
-    addFormatTest(root);
-#endif
-#if !UCONFIG_NO_BREAK_ITERATION
-    addBreakIter(root);
-#endif
-#if !UCONFIG_NO_COLLATION
-    addCollTest(root);
-#endif
+    addUCsdetTest(root);
     addComplexTest(root);
-#if !UCONFIG_NO_TRANSLITERATION
-    addUTransTest(root);
-#endif
     addUSetTest(root);
-    addTestDeprecatedAPI(root);
 #if !UCONFIG_NO_IDNA
     addUStringPrepTest(root);
     addIDNATest(root);
 #endif
-    addHeapMutexTest(root);
-    addUTraceTest(root);
-
+#if !UCONFIG_NO_REGULAR_EXPRESSIONS
+    addURegexTest(root);
+#endif
+#if !UCONFIG_NO_BREAK_ITERATION
+    addBreakIter(root);
+#endif
+#if !UCONFIG_NO_FORMATTING
+    addFormatTest(root);
+#endif
+#if !UCONFIG_NO_COLLATION
+    addCollTest(root);
+#endif
+#if !UCONFIG_NO_TRANSLITERATION
+    addUTransTest(root);
+#endif
+#if !UCONFIG_NO_REGULAR_EXPRESSIONS && !UCONFIG_NO_NORMALIZATION
+    addUSpoofTest(root);
+#endif
+    addPUtilTest(root);
+#if !UCONFIG_NO_FORMATTING
+    addGendInfoForTest(root);
+#endif
 }
-
