@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT:
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2009, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -22,7 +22,11 @@
  */
 
 #include <stdlib.h>
+
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/ucol.h"
 #include "unicode/uloc.h"
 #include "cintltst.h"
@@ -115,7 +119,7 @@ static void TestTertiary( )
     UErrorCode status = U_ZERO_ERROR;
     myCollation = ucol_open("ja_JP", &status);
     if(U_FAILURE(status)){
-        log_err("ERROR: in creation of rule based collator: %s\n", myErrorName(status));
+        log_err_status(status, "ERROR: in creation of rule based collator: %s\n", myErrorName(status));
         return;
     }
     log_verbose("Testing Kanna(Japan) Collation with Tertiary strength\n");
@@ -136,7 +140,7 @@ static void TestBase()
     myCollation = ucol_open("ja_JP", &status);
     if (U_FAILURE(status))
     {
-        log_err("ERROR: in creation of rule based collator: %s\n",
+        log_err_status(status, "ERROR: in creation of rule based collator: %s\n",
             myErrorName(status));
         return;
     }
@@ -157,7 +161,7 @@ static void TestPlainDakutenHandakuten(void)
     myCollation = ucol_open("ja_JP", &status);
     if (U_FAILURE(status))
     {
-        log_err("ERROR: in creation of rule based collator: %s\n",
+        log_err_status(status, "ERROR: in creation of rule based collator: %s\n",
             myErrorName(status));
         return;
     }
@@ -181,7 +185,7 @@ static void TestSmallLarge(void)
     myCollation = ucol_open("ja_JP", &status);
     if (U_FAILURE(status))
     {
-        log_err("ERROR: in creation of rule based collator: %s\n",
+        log_err_status(status, "ERROR: in creation of rule based collator: %s\n",
             myErrorName(status));
         return;
     }
@@ -206,7 +210,7 @@ static void TestKatakanaHiragana(void)
     myCollation = ucol_open("ja_JP", &status);
     if (U_FAILURE(status))
     {
-        log_err("ERROR: in creation of rule based collator: %s\n",
+        log_err_status(status, "ERROR: in creation of rule based collator: %s\n",
             myErrorName(status));
         return;
     }
@@ -232,12 +236,13 @@ static void TestChooonKigoo(void)
     myCollation = ucol_open("ja_JP", &status);
     if (U_FAILURE(status))
     {
-        log_err("ERROR: in creation of rule based collator: %s\n",
+        log_err_status(status, "ERROR: in creation of rule based collator: %s\n",
             myErrorName(status));
         return;
     }
 
     log_verbose("Testing Japanese Choo-on Kigoo Characters Collation\n");
+    ucol_setAttribute(myCollation, UCOL_STRENGTH, UCOL_QUATERNARY, &status);
     ucol_setAttribute(myCollation, UCOL_CASE_LEVEL, UCOL_ON, &status);
     for (i = 0; i < 7 ; i++) {
         doTest(myCollation, testChooonKigooCases[i], testChooonKigooCases[i + 1],
@@ -246,3 +251,5 @@ static void TestChooonKigoo(void)
 
     ucol_close(myCollation);
 }
+
+#endif /* #if !UCONFIG_NO_COLLATION */
