@@ -21,6 +21,9 @@
 #ifndef _CENCOLLTST
 #define _CENCOLLTST
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
 
 #include "cintltst.h"
 
@@ -36,5 +39,6 @@ static    void TestSecondary(void);
 static    void TestTertiary(void);
 
 
+#endif /* #if !UCONFIG_NO_COLLATION */
 
 #endif
