@@ -1,5 +1,5 @@
 /***********************************************************************
- * Copyright (c) 1997-2013, International Business Machines Corporation
+ * Copyright (c) 1997-2014, International Business Machines Corporation
  * and others. All Rights Reserved.
  ***********************************************************************/
 
@@ -240,12 +240,17 @@ public: // package
 
     void TestCloneLocale(void);
 
+    void TestHebrewMonthValidation(void);
+
     /*
      * utility methods for TestIslamicUmAlQura
      */
     void setAndTestCalendar(Calendar* cal, int32_t initMonth, int32_t initDay, int32_t initYear, UErrorCode& status);
     void setAndTestWholeYear(Calendar* cal, int32_t startYear, UErrorCode& status);
 
+    void TestWeekData(void);
+
+    void TestAddAcrossZoneTransition(void);
 };
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
