@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2013, International Business Machines Corporation and
+ * Copyright (c) 1997-2014, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -150,6 +150,8 @@ public: // package
      */
     virtual void TestLocaleDateFormat(void);
 
+    virtual void TestFormattingLocaleTimeSeparator(void);
+
     virtual void TestDateFormatCalendar(void);
 
     virtual void TestSpaceParsing(void);
@@ -214,6 +216,10 @@ public:
     
     void TestHostClone(void);
 
+    void TestHebrewClone(void);
+
+    void TestDateFormatSymbolsClone(void);
+
     void TestTimeZoneDisplayName(void);
 
     void TestRoundtripWithCalendar(void);
@@ -231,6 +237,19 @@ public:
 
     void TestDateFormatLeniency();
 
+    void TestParseMultiPatternMatch();
+
+    void TestParseLeniencyAPIs();
+
+    // test override NumberFormat
+    void TestNumberFormatOverride();
+    void TestCreateInstanceForSkeleton();
+    void TestCreateInstanceForSkeletonDefault();
+    void TestCreateInstanceForSkeletonWithCalendar();
+    void TestDFSCreateForLocaleNonGregorianLocale();
+    void TestDFSCreateForLocaleWithCalendarInLocale();
+    void TestChangeCalendar();
+
 private:
     UBool showParse(DateFormat &format, const UnicodeString &formattedString);
 
