@@ -1,18 +1,18 @@
 /**
  *******************************************************************************
- * Copyright (C) 2001-2003, International Business Machines Corporation and    *
- * others. All Rights Reserved.                                                *
- *******************************************************************************
- *
+ * Copyright (C) 2001-2013, International Business Machines Corporation and
+ * others. All Rights Reserved.
  *******************************************************************************
  */
 
+#include "utypeinfo.h"  // for 'typeid' to work
+
 #include "unicode/utypes.h"
 
 #if !UCONFIG_NO_SERVICE
 
 #include "icusvtst.h"
-#include "iculserv.h"
+#include "servloc.h"
 #include <stdio.h>
 
 
@@ -24,30 +24,39 @@ class WrongListener : public EventListener {
 
 class ICUNSubclass : public ICUNotifier {
     public:
-    UBool acceptsListener(const EventListener& l) const {
+    UBool acceptsListener(const EventListener& /*l*/) const {
         return TRUE;
         // return l instanceof MyListener;
     }
 
-    virtual void notifyListener(EventListener& l) const {
+    virtual void notifyListener(EventListener& /*l*/) const {
     }
 };
 
+// This factory does nothing
+class LKFSubclass0 : public LocaleKeyFactory {
+public:
+        LKFSubclass0()
+                : LocaleKeyFactory(VISIBLE, "LKFSubclass0")
+        {
+        }
+};
+
 class LKFSubclass : public LocaleKeyFactory {
-	Hashtable table;
+    Hashtable table;
 
     public:
     LKFSubclass(UBool visible) 
         : LocaleKeyFactory(visible ? VISIBLE : INVISIBLE, "LKFSubclass")
     {
-		UErrorCode status = U_ZERO_ERROR;
-		table.put("en_US", this, status);
+        UErrorCode status = U_ZERO_ERROR;
+        table.put("en_US", this, status);
     }
 
     protected:
-	virtual const Hashtable* getSupportedIDs(UErrorCode &/*status*/) const {
-		return &table;
-	}
+    virtual const Hashtable* getSupportedIDs(UErrorCode &/*status*/) const {
+        return &table;
+    }
 };
 
 class Integer : public UObject {
@@ -62,22 +71,22 @@ class Integer : public UObject {
     virtual ~Integer() {
     }
 
-    virtual UBool operator==(const UObject& other) const 
-    {
-        return other.getDynamicClassID() == getStaticClassID() &&
-            _val == ((Integer&)other)._val;
-    }
-
     public:
     /**
      * UObject boilerplate.
      */
+    static UClassID getStaticClassID() { 
+        return (UClassID)&fgClassID;
+    }
+
     virtual UClassID getDynamicClassID() const {
         return getStaticClassID();
     }
 
-    static UClassID getStaticClassID() { 
-        return (UClassID)&fgClassID;
+    virtual UBool operator==(const UObject& other) const 
+    {
+        return typeid(*this) == typeid(other) &&
+            _val == ((Integer&)other)._val;
     }
 
     public:
@@ -107,8 +116,9 @@ class TestIntegerService : public ICUService {
 
     virtual ICUServiceFactory* createSimpleFactory(UObject* obj, const UnicodeString& id, UBool visible, UErrorCode& status) 
     {
-        if (U_SUCCESS(status) && obj && obj->getDynamicClassID() == Integer::getStaticClassID()) {
-            return new SimpleFactory((Integer*)obj, id, visible);
+        Integer* i;
+        if (U_SUCCESS(status) && obj && (i = dynamic_cast<Integer*>(obj)) != NULL) {
+            return new SimpleFactory(i, id, visible);
         }
         return NULL;
     }
@@ -147,13 +157,15 @@ UnicodeString append(UnicodeString& result, const UObject* obj)
     if (obj == NULL) {
         result.append("NULL");
     } else {
-        UClassID id = obj->getDynamicClassID();
-        if (id == UnicodeString::getStaticClassID()) {
-            result.append(*(UnicodeString*)obj);
-        } else if (id == Locale::getStaticClassID()) {
-            result.append(((Locale*)obj)->getName());
-        } else if (id == Integer::getStaticClassID()) {
-            sprintf(buffer, "%d", ((Integer*)obj)->_val);
+        const UnicodeString* s;
+        const Locale* loc;
+        const Integer* i;
+        if ((s = dynamic_cast<const UnicodeString*>(obj)) != NULL) {
+            result.append(*s);
+        } else if ((loc = dynamic_cast<const Locale*>(obj)) != NULL) {
+            result.append(loc->getName());
+        } else if ((i = dynamic_cast<const Integer*>(obj)) != NULL) {
+            sprintf(buffer, "%d", (int)i->_val);
             result.append(buffer);
         } else {
             sprintf(buffer, "%p", (const void*)obj);
@@ -269,7 +281,7 @@ ICUServiceTest::confirmStringsEqual(const UnicodeString& message, const UnicodeS
     if (equ) {
         logln(temp);
     } else {
-        errln(temp);
+        dataerrln(temp);
     }
 }
 
@@ -300,7 +312,7 @@ void
 ICUServiceTest::msgstr(const UnicodeString& message, UObject* obj, UBool err)
 {
     if (obj) {
-	UnicodeString* str = (UnicodeString*)obj;
+    UnicodeString* str = (UnicodeString*)obj;
         logln(message + *str);
         delete str;
     } else if (err) {
@@ -317,7 +329,7 @@ ICUServiceTest::testAPI_One()
     // register an object with one locale, 
     // search for an object with a more specific locale
     // should return the original object
-	UErrorCode status = U_ZERO_ERROR;
+    UErrorCode status = U_ZERO_ERROR;
     Integer* singleton0 = new Integer(0);
     service.registerInstance(singleton0, "en_US", status);
     {
@@ -442,11 +454,11 @@ ICUServiceTest::testAPI_One()
         confirmEqual("17) get invisible", result, singleton5);
         delete result;
     }
-	
+    
     // should not be able to locate invisible services
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector ids(uhash_deleteUnicodeString, uhash_compareUnicodeString, status);
+        UVector ids(uprv_deleteUObject, uhash_compareUnicodeString, status);
         service.getVisibleIDs(ids, status);
         UnicodeString target = "en_US_BAR";
         confirmBoolean("18) find invisible", !ids.contains(&target));
@@ -460,6 +472,25 @@ ICUServiceTest::testAPI_One()
 /*
  ******************************************************************
  */
+class TestStringSimpleKeyService : public ICUService {
+public:
+ 
+        virtual ICUServiceFactory* createSimpleFactory(UObject* obj, const UnicodeString& id, UBool visible, UErrorCode& status) 
+    {
+                // We could put this type check into ICUService itself, but we'd still
+                // have to implement cloneInstance.  Otherwise we could just tell the service
+                // what the object type is when we create it, and the default implementation
+                // could handle everything for us.  Phooey.
+        if (obj && dynamic_cast<UnicodeString*>(obj) != NULL) {
+                        return ICUService::createSimpleFactory(obj, id, visible, status);
+        }
+        return NULL;
+    }
+
+    virtual UObject* cloneInstance(UObject* instance) const {
+        return instance ? new UnicodeString(*(UnicodeString*)instance) : NULL;
+    }
+};
 
 class TestStringService : public ICUService {
     public:
@@ -469,8 +500,9 @@ class TestStringService : public ICUService {
 
   virtual ICUServiceFactory* createSimpleFactory(UObject* obj, const UnicodeString& id, UBool visible, UErrorCode& /* status */) 
     {
-        if (obj && obj->getDynamicClassID() == UnicodeString::getStaticClassID()) {
-            return new SimpleFactory((UnicodeString*)obj, id, visible);
+        UnicodeString* s;
+        if (obj && (s = dynamic_cast<UnicodeString*>(obj)) != NULL) {
+            return new SimpleFactory(s, id, visible);
         }
         return NULL;
     }
@@ -497,14 +529,14 @@ class AnonymousStringFactory : public ICUServiceFactory
         return result;
     }
 
-    virtual UClassID getDynamicClassID() const {
-        return getStaticClassID();
-    }
-
     static UClassID getStaticClassID() {
         return (UClassID)&fgClassID;
     }
 
+    virtual UClassID getDynamicClassID() const {
+        return getStaticClassID();
+    }
+
     private:
     static const char fgClassID;
 };
@@ -519,7 +551,7 @@ class TestMultipleKeyStringFactory : public ICUServiceFactory {
     public:
     TestMultipleKeyStringFactory(const UnicodeString ids[], int32_t count, const UnicodeString& factoryID)
         : _status(U_ZERO_ERROR)
-        , _ids(uhash_deleteUnicodeString, uhash_compareUnicodeString, count, _status)
+        , _ids(uprv_deleteUObject, uhash_compareUnicodeString, count, _status)
         , _factoryID(factoryID + ": ") 
     {
         for (int i = 0; i < count; ++i) {
@@ -532,17 +564,17 @@ class TestMultipleKeyStringFactory : public ICUServiceFactory {
 
     UObject* create(const ICUServiceKey& key, const ICUService* /* service */, UErrorCode& status) const {
         if (U_FAILURE(status)) {
-	    return NULL;
+        return NULL;
         }
         UnicodeString temp;
         key.currentID(temp);
         if (U_SUCCESS(_status)) {
-	    if (_ids.contains(&temp)) {
+        if (_ids.contains(&temp)) {
                 return new UnicodeString(_factoryID + temp);
-	    }
+        }
         } else {
-	    status = _status;
-	}
+        status = _status;
+    }
         return NULL;
     }
 
@@ -573,14 +605,14 @@ class TestMultipleKeyStringFactory : public ICUServiceFactory {
         return result;
     }
 
-    virtual UClassID getDynamicClassID() const {
-        return getStaticClassID();
-    }
-
     static UClassID getStaticClassID() {
         return (UClassID)&fgClassID;
     }
 
+    virtual UClassID getDynamicClassID() const {
+        return getStaticClassID();
+    }
+
     private:
     static const char fgClassID;
 };
@@ -590,7 +622,7 @@ const char TestMultipleKeyStringFactory::fgClassID = '\0';
 void 
 ICUServiceTest::testAPI_Two()
 {
-	UErrorCode status = U_ZERO_ERROR;
+    UErrorCode status = U_ZERO_ERROR;
     TestStringService service;
     service.registerFactory(new AnonymousStringFactory(), status);
 
@@ -648,7 +680,7 @@ ICUServiceTest::testAPI_Two()
     // iterate over the visual ids returned by the multiple factory
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector ids(uhash_deleteUnicodeString, uhash_compareUnicodeString, 0, status);
+        UVector ids(uprv_deleteUObject, uhash_compareUnicodeString, 0, status);
         service.getVisibleIDs(ids, status);
         for (int i = 0; i < ids.size(); ++i) {
             const UnicodeString* id = (const UnicodeString*)ids[i];
@@ -667,8 +699,8 @@ ICUServiceTest::testAPI_Two()
     // iterate over the display names
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector names(userv_deleteStringPair, NULL, status);
-        service.getDisplayNames(names, Locale::getGerman(), status);
+        UVector names(status);
+        service.getDisplayNames(names, status);
         for (int i = 0; i < names.size(); ++i) {
             const StringPair* pair = (const StringPair*)names[i];
             logln("  " + pair->displayName + " --> " + pair->id);
@@ -707,13 +739,13 @@ ICUServiceTest::testAPI_Two()
     // Rad dude's surfer gal 'replaces' Later's surfer gal
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector names(userv_deleteStringPair, NULL, status);
+        UVector names(status);
         service.getDisplayNames(names, Locale("es"), status);
         for (int i = 0; i < names.size(); ++i) {
             const StringPair* pair = (const StringPair*)names[i];
             logln("  " + pair->displayName + " --> " + pair->id);
         }
-        confirmIdentical("26) display names", names.size(), 7);
+        confirmIdentical("29) display names", names.size(), 7);
     }
 
     // we should get the display name corresponding to the actual id
@@ -769,7 +801,7 @@ ICUServiceTest::testAPI_Two()
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector ids(uhash_deleteUnicodeString, uhash_compareUnicodeString, 0, status);
+        UVector ids(uprv_deleteUObject, uhash_compareUnicodeString, 0, status);
         service.getVisibleIDs(ids, status);
         for (int i = 0; i < ids.size(); ++i) {
             const UnicodeString* id = (const UnicodeString*)ids[i];
@@ -858,14 +890,14 @@ void
 ICUServiceTest::testRBF()
 {
     // resource bundle factory.
-	UErrorCode status = U_ZERO_ERROR;
+    UErrorCode status = U_ZERO_ERROR;
     TestStringService service;
     service.registerFactory(new ICUResourceBundleFactory(), status);
 
     // list all of the resources 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector ids(uhash_deleteUnicodeString, uhash_compareUnicodeString, 0, status);
+        UVector ids(uprv_deleteUObject, uhash_compareUnicodeString, 0, status);
         service.getVisibleIDs(ids, status);
         logln("all visible ids:");
         for (int i = 0; i < ids.size(); ++i) {
@@ -878,7 +910,7 @@ ICUServiceTest::testRBF()
     // this should be fast since the display names were cached.
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector names(userv_deleteStringPair, NULL, status);
+        UVector names(status);
         service.getDisplayNames(names, Locale::getGermany(), status);
         logln("service display names for de_DE");
         for (int i = 0; i < names.size(); ++i) {
@@ -904,7 +936,7 @@ ICUServiceTest::testRBF()
             logln(UnicodeString("\n  --- ") + idNames[i] + " ---");
             {
                 UErrorCode status = U_ZERO_ERROR;
-                UVector names(userv_deleteStringPair, NULL, status);
+                UVector names(status);
                 service.getDisplayNames(names, idNames[i], status);
                 for (int i = 0; i < names.size(); ++i) {
                     const StringPair* pair = (const StringPair*)names[i];
@@ -918,11 +950,10 @@ ICUServiceTest::testRBF()
 
 class SimpleListener : public ServiceListener {
     ICUServiceTest* _test;
-    int32_t _n;
     UnicodeString _name;
 
     public:
-    SimpleListener(ICUServiceTest* test, const UnicodeString& name) : _test(test), _n(0), _name(name) {}
+    SimpleListener(ICUServiceTest* test, const UnicodeString& name) : _test(test), _name(name) {}
 
     virtual void serviceChanged(const ICUService& service) const {
         UnicodeString serviceName = "listener ";
@@ -1002,7 +1033,7 @@ class TestStringLocaleService : public ICULocaleService {
 };
 
 void ICUServiceTest::testLocale() {
-	UErrorCode status = U_ZERO_ERROR;
+    UErrorCode status = U_ZERO_ERROR;
     TestStringLocaleService service;
 
     UnicodeString* root = new UnicodeString("root");
@@ -1014,7 +1045,7 @@ void ICUServiceTest::testLocale() {
     service.registerInstance(root, "", status);
     service.registerInstance(german, "de", status);
     service.registerInstance(germany, Locale::getGermany(), status);
-    service.registerInstance(japanese, "ja", status);
+    service.registerInstance(japanese, (UnicodeString)"ja", TRUE, status);
     service.registerInstance(japan, Locale::getJapan(), status);
 
     {
@@ -1092,7 +1123,7 @@ void ICUServiceTest::testLocale() {
     }
 
     {
-		UErrorCode status = U_ZERO_ERROR;
+        UErrorCode status = U_ZERO_ERROR;
         UnicodeString english = "en";
         Locale localeResult;
         UnicodeString result;
@@ -1132,7 +1163,7 @@ void ICUServiceTest::testLocale() {
 
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector ids(uhash_deleteUnicodeString, uhash_compareUnicodeString, 0, status);
+        UVector ids(uprv_deleteUObject, uhash_compareUnicodeString, 0, status);
         service.getVisibleIDs(ids, status);
         logln("all visible ids:");
         for (int i = 0; i < ids.size(); ++i) {
@@ -1144,7 +1175,7 @@ void ICUServiceTest::testLocale() {
     Locale::setDefault(loc, status);
     {
         UErrorCode status = U_ZERO_ERROR;
-        UVector ids(uhash_deleteUnicodeString, uhash_compareUnicodeString, 0, status);
+        UVector ids(uprv_deleteUObject, uhash_compareUnicodeString, 0, status);
         service.getVisibleIDs(ids, status);
         logln("all visible ids:");
         for (int i = 0; i < ids.size(); ++i) {
@@ -1164,7 +1195,7 @@ void ICUServiceTest::testLocale() {
         UErrorCode status = U_ZERO_ERROR;
         StringEnumeration* locales = service.getAvailableLocales();
         if (locales) {
-            confirmIdentical("test available locales", locales->count(status), 5);
+            confirmIdentical("test available locales", locales->count(status), 6);
             logln("locales: ");
             {
                 const char* p;
@@ -1184,7 +1215,7 @@ class WrapFactory : public ICUServiceFactory {
     public:
     static const UnicodeString& getGreetingID() {
       if (greetingID == NULL) {
-	greetingID = new UnicodeString("greeting");
+    greetingID = new UnicodeString("greeting");
       }
       return *greetingID;
     }
@@ -1225,14 +1256,14 @@ class WrapFactory : public ICUServiceFactory {
     /**
      * UObject boilerplate.
      */
-    virtual UClassID getDynamicClassID() const {
-        return getStaticClassID();
-    }
-
     static UClassID getStaticClassID() { 
         return (UClassID)&fgClassID;
     }
 
+    virtual UClassID getDynamicClassID() const {
+        return getStaticClassID();
+    }
+
     private:
     static const char fgClassID;
     static UnicodeString* greetingID;
@@ -1246,7 +1277,7 @@ ICUServiceTest::testWrapFactory()
 {
     UnicodeString* greeting = new UnicodeString("Hello There");
     UnicodeString greetingID = "greeting";
-	UErrorCode status = U_ZERO_ERROR;
+    UErrorCode status = U_ZERO_ERROR;
     TestStringService service;
     service.registerInstance(greeting, greetingID, status);
 
@@ -1303,18 +1334,45 @@ void ICUServiceTest::testCoverage()
     }
 
     // ICUService
-    TestStringService service;
-    service.registerFactory(sf, status);
+        {
+                TestStringService service;
+                service.registerFactory(sf,     status);
 
-    {
-      UnicodeString* result = (UnicodeString*)service.get("object", status);
-      if (result) {
-	logln("object is: " + *result);
-	delete result;
-      } else {
-	errln("could not get object");
-      }
-    }
+                {
+                        UnicodeString* result   = (UnicodeString*)service.get("object", status);
+                        if (result) {
+                                logln("object is: "     + *result);
+                                delete result;
+                        }       else {
+                                errln("could not get object");
+                        }
+                }
+        }
+  }
+  
+  // ICUServiceKey
+  {
+      UErrorCode status = U_ZERO_ERROR;
+          UnicodeString* howdy = new UnicodeString("Howdy");
+
+          TestStringSimpleKeyService service;
+          service.registerInstance(howdy, "Greetings", status);
+          {
+                  UnicodeString* result = (UnicodeString*)service.get("Greetings",      status);
+                  if (result) {
+                          logln("object is: "   + *result);
+                          delete result;
+                  }     else {
+                          errln("could not get object");
+                  }
+          }
+
+      UVector ids(uprv_deleteUObject, uhash_compareUnicodeString, status);
+          // yuck, this is awkward to use.  All because we pass null in an overload.
+          // TODO: change this.
+          UnicodeString str("Greet");
+      service.getVisibleIDs(ids, &str, status);
+      confirmIdentical("no fallback of greet", ids.size(), 0);
   }
 
   // ICULocaleService
@@ -1374,15 +1432,40 @@ void ICUServiceTest::testCoverage()
     }
     delete obj;
     delete key;
+
+        key = LocaleKey::createWithCanonicalFallback(&primary, &fallback, 123, status);
+        if (U_SUCCESS(status)) {
+                UnicodeString str;
+                key->currentDescriptor(str);
+                key->parsePrefix(str);
+                if (str != "123") {
+                        errln("did not get expected prefix");
+                }
+                delete key;
+        }
+
+        // coverage, getSupportedIDs is either overridden or the calling method is
+        LKFSubclass0 lkFactory;
+        Hashtable table0;
+        lkFactory.updateVisibleIDs(table0, status);
+        if (table0.count() != 0) {
+                errln("LKF returned non-empty hashtable");
+        }
+
+
+        // ResourceBundleFactory
+    key = LocaleKey::createWithCanonicalFallback(&primary, &fallback, status);
+        ICUResourceBundleFactory rbf;
+        UObject* icurb = rbf.create(*key, NULL, status);
+        if (icurb != NULL) {
+                logln("got resource bundle for key");
+                delete icurb;
+        }
+        delete key;
   }
 
-
-#if 0
-  // ResourceBundleFactory
-  ICUResourceBundleFactory rbf = new ICUResourceBundleFactory();
-  logln("RB: " + rbf.create(lkey, null));
-
-  // ICUNotifier
+ #if 0
+ // ICUNotifier
   ICUNotifier nf = new ICUNSubclass();
   try {
     nf.addListener(null);
