@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2003, International Business Machines Corporation and
+ * Copyright (c) 1997-2015, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -76,6 +76,9 @@ public:
    /* Test getting keyword values */
    void TestKeywordVariantParsing(void);
 
+   /* Test setting keyword values */
+   void TestSetKeywordValue(void);
+
    /* Test getting the locale base name */
    void TestGetBaseName(void);
     
@@ -85,12 +88,22 @@ public:
 
     void TestSetIsBogus(void);
 
-    void TestGetLocale();
+    void TestGetLocale(void);
+
+    void TestVariantWithOutCountry(void);
+
+    void TestCanonicalization(void);
 
 #if !UCONFIG_NO_FORMATTING
     static UDate date(int32_t y, int32_t m, int32_t d, int32_t hr = 0, int32_t min = 0, int32_t sec = 0);
 #endif
 
+    void TestCurrencyByDate(void);
+
+    void TestGetVariantWithKeywords(void);
+    void TestIsRightToLeft();
+    void TestBug11421();
+
 private:
     void _checklocs(const char* label,
                     const char* req,
@@ -157,7 +170,12 @@ private:
         DVAR_NO,
         DNAME_NO
     };
+
+#if !UCONFIG_NO_COLLATION
+    /**
+     * Check on registered collators.
+     * @param expectExtra if non-null, the locale ID of an 'extra' locale that is registered.
+     */
+    void checkRegisteredCollators(const char *expectExtra = NULL);
+#endif
 };
-
-
-
