@@ -1,12 +1,15 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2003, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
 #ifndef _INTLTESTNUMBERFORMATAPI
 #define _INTLTESTNUMBERFORMATAPI
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
 
 #include "intltest.h"
 
@@ -22,6 +25,9 @@ private:
      * executes tests of API functions, see detailed comments in source code
      **/
     void testAPI(/* char* par */);
+    void testRegistration();
 };
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif
