@@ -1,6 +1,6 @@
 /*
 **********************************************************************
-*   Copyright (C) 1999-2006, International Business Machines
+*   Copyright (C) 1999-2011, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 **********************************************************************
 *   Date        Name        Description
@@ -334,9 +334,11 @@ private:
 
     void TestAnyX(void);
 
+    void TestAny(void);
+
     void TestSourceTargetSet(void);
 
-    void TestRuleWhitespace(void);
+    void TestPatternWhiteSpace(void);
 
     void TestAllCodepoints(void);
 
@@ -344,6 +346,12 @@ private:
 
     void TestAlternateSyntax(void);
 
+    void TestRuleStripping(void);
+
+    void TestHalfwidthFullwidth(void);
+
+    void TestThai(void);
+
     /**
      * Tests the multiple-pass syntax
      */
