@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2003, International Business Machines Corporation and
+ * Copyright (c) 1997-2014, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -11,25 +11,6 @@
 #include "intltest.h"
 #include "mutex.h"
 
-/*
-    Test multithreading.   Of course we need a thread class first..
-    this wrapper has a ported implementation.
- */
-
-class SimpleThread
-{
-public:
-    SimpleThread();
-    virtual ~SimpleThread();
-    int32_t start(void); // start the thread
-public: // should be private, but then we couldn't be asocial.
-    virtual void run(void) = 0; // Override this to provide some real implementation
-private:
-    void *fImplementation;
-
-public:
-    static void sleep(int32_t millis); // probably shouldn't go here but oh well. 
-};
 
 
 /**
@@ -47,6 +28,12 @@ public:
      * test that threads even work
      **/
     void TestThreads(void);
+
+	/**
+     * test that arabic shaping can work in threads
+     **/
+    void TestArabicShapingThreads(void);
+	
     /**
      * test that mutexes work 
      **/
@@ -57,7 +44,11 @@ public:
      **/
     void TestThreadedIntl(void);
 #endif
-  void TestCollators(void);
+    void TestCollators(void);
+    void TestString();
+    void TestAnyTranslit();
+    void TestConditionVariables();
+    void TestUnifiedCache();
 
 };
 
