@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2003, International Business Machines Corporation and
+ * Copyright (c) 1997-2013, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
@@ -8,14 +8,20 @@
 #include "intltest.h"
 
 /** Helper function for TestUnicodeData */
-U_CAPI void U_CALLCONV unicodeDataLineFn(void *context,
+U_CFUNC void U_CALLCONV unicodeDataLineFn(void *context,
                               char *fields[][2], int32_t fieldCount,
                               UErrorCode *pErrorCode);
 
-U_CAPI void U_CALLCONV
-derivedCorePropsLineFn(void *context,
-                       char *fields[][2], int32_t fieldCount,
-                       UErrorCode *pErrorCode);
+U_CFUNC void U_CALLCONV
+derivedPropsLineFn(void *context,
+                   char *fields[][2], int32_t fieldCount,
+                   UErrorCode *pErrorCode);
+
+U_NAMESPACE_BEGIN
+
+class Hashtable;
+
+U_NAMESPACE_END
 
 /** 
  * Test API and functionality of class Unicode
@@ -28,6 +34,11 @@ public:
     void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
 
     void TestAdditionalProperties();
+    void TestBinaryValues();
+    void TestConsistency();
+    void TestPatternProperties();
+    void TestScriptMetadata();
+    void TestBidiPairedBracketType();
 
 private:
 
@@ -36,10 +47,14 @@ private:
                               UErrorCode *pErrorCode);
 
     friend void U_CALLCONV
-    derivedCorePropsLineFn(void *context,
+    derivedPropsLineFn(void *context,
                            char *fields[][2], int32_t fieldCount,
                            UErrorCode *pErrorCode);
 
-    UnicodeSet derivedCoreProps[30];
-};
+    UnicodeSet derivedProps[30];
+    U_NAMESPACE_QUALIFIER Hashtable *unknownPropertyNames;
 
+    UBool compareUSets(const UnicodeSet &a, const UnicodeSet &b,
+                       const char *a_name, const char *b_name,
+                       UBool diffIsError);
+};
