@@ -1,6 +1,6 @@
 /*
  **********************************************************************
- *   Copyright (C) 2004-2010, International Business Machines
+ *   Copyright (C) 2004-2014, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  **********************************************************************
  *   file name:  filetst.c
@@ -22,6 +22,8 @@
 
 const char *STANDARD_TEST_FILE = "iotest-c.txt";
 
+const char *STANDARD_TEST_LOCALE = "en_US_POSIX";
+
 
 #if !UCONFIG_NO_FORMATTING
 static void TestFileFromICU(UFILE *myFile) {
@@ -98,7 +100,7 @@ static void TestFileFromICU(UFILE *myFile) {
     fprintf(u_fgetfile(myFile), "\tNormal fprintf count value: n=%d\n", (int)*n); /* Should be 27 as stated later on. */
 
     u_fclose(myFile);
-    myFile = u_fopen(STANDARD_TEST_FILE, "r", "en_US_POSIX", NULL);
+    myFile = u_fopen(STANDARD_TEST_FILE, "r", STANDARD_TEST_LOCALE, NULL);
 
     if (myFile == NULL) {
         log_err("Can't read test file.");
@@ -237,7 +239,7 @@ static void TestFileFromICU(UFILE *myFile) {
     myString[2] = '!';
     myString[3] = '!';
     u_austrncpy(myString, myUString, sizeof(myUString)/sizeof(*myUString));
-    if (myString == NULL || strcmp(myString, "\t\n") != 0) {
+    if (strcmp(myString, "\t\n") != 0) {
         log_err("u_fgets got \"%s\"\n", myString);
     }
 
@@ -245,7 +247,7 @@ static void TestFileFromICU(UFILE *myFile) {
         log_err("u_fgets did not return myUString\n");
     }
     u_austrncpy(myString, myUString, sizeof(myUString)/sizeof(*myUString));
-    if (myString == NULL || strcmp(myString, "Pointer to integer (Count) %n: n=1  n=1\n") != 0) {
+    if (strcmp(myString, "Pointer to integer (Count) %n: n=1  n=1\n") != 0) {
         log_err("u_fgets got \"%s\"\n", myString);
     }
 
@@ -253,7 +255,7 @@ static void TestFileFromICU(UFILE *myFile) {
         log_err("u_fgets did not return myUString\n");
     }
     u_austrncpy(myString, myUString, sizeof(myUString)/sizeof(*myUString));
-    if (myString == NULL || strcmp(myString, "Pointer to integer Value: 37\n") != 0) {
+    if (strcmp(myString, "Pointer to integer Value: 37\n") != 0) {
         log_err("u_fgets got \"%s\"\n", myString);
     }
 
@@ -261,7 +263,7 @@ static void TestFileFromICU(UFILE *myFile) {
         log_err("u_fgets did not return myUString\n");
     }
     u_austrncpy(myString, myUString, sizeof(myUString)/sizeof(*myUString));
-    if (myString == NULL || strcmp(myString, "This is a long test123456789012345678901234567890123456789012345678901234567890\n") != 0) {
+    if (strcmp(myString, "This is a long test123456789012345678901234567890123456789012345678901234567890\n") != 0) {
         log_err("u_fgets got \"%s\"\n", myString);
     }
 
@@ -273,7 +275,7 @@ static void TestFileFromICU(UFILE *myFile) {
         log_err("u_fgets did not return myUString\n");
     }
     u_austrncpy(myString, myUString, sizeof(myUString)/sizeof(*myUString));
-    if (myString == NULL || strcmp(myString, "") != 0) {
+    if (strcmp(myString, "") != 0) {
         log_err("u_fgets got \"%s\"\n", myString);
     }
 
@@ -281,7 +283,7 @@ static void TestFileFromICU(UFILE *myFile) {
         log_err("u_fgets did not return myUString\n");
     }
     u_austrncpy(myString, myUString, sizeof(myUString)/sizeof(*myUString));
-    if (myString == NULL || strcmp(myString, "\t") != 0) {
+    if (strcmp(myString, "\t") != 0) {
         log_err("u_fgets got \"%s\"\n", myString);
     }
 
@@ -310,22 +312,26 @@ static void TestFileFromICU(UFILE *myFile) {
 }
 
 static void TestFile(void) {
-/*    FILE *standardFile;*/
 
     log_verbose("Testing u_fopen\n");
-    TestFileFromICU(u_fopen(STANDARD_TEST_FILE, "w", "en_US_POSIX", NULL));
+    TestFileFromICU(u_fopen(STANDARD_TEST_FILE, "w", STANDARD_TEST_LOCALE, NULL));
+}
+
+static void TestFinit(void) {
+    FILE *standardFile;
 
-/* Don't know how to make this work without stdout or stderr */
-/*
     log_verbose("Testing u_finit\n");
-    standardFile = fopen(STANDARD_TEST_FILE, "wb");
-    TestFileFromICU(u_finit(standardFile, NULL, NULL));
+    standardFile = fopen(STANDARD_TEST_FILE, "w");
+    TestFileFromICU(u_finit(standardFile, STANDARD_TEST_LOCALE, NULL));
     fclose(standardFile);
+}
+
+static void TestFadopt(void) {
+    FILE *standardFile;
 
     log_verbose("Testing u_fadopt\n");
-    standardFile = fopen(STANDARD_TEST_FILE, "wb");
-    TestFileFromICU(u_fadopt(standardFile, NULL, NULL));
-*/
+    standardFile = fopen(STANDARD_TEST_FILE, "w");
+    TestFileFromICU(u_fadopt(standardFile, STANDARD_TEST_LOCALE, NULL));
 }
 #endif
 
@@ -909,22 +915,86 @@ static void TestCodepage(void) {
 
 }
 
+static void TestCodepageFlush(void) {
+#if UCONFIG_NO_LEGACY_CONVERSION || UCONFIG_NO_FORMATTING
+  log_verbose("Skipping, legacy conversion or formatting is disabled.");
+#else
+  UChar utf16String[] = { 0x39, 0x39, 0x39, 0x20, 0x65E0, 0x6CD6, 0x5728, 0x0000 };
+  uint8_t inBuf[200];
+  size_t inLen =0;
+  const char *enc = "IBM-1388"; /* GBK EBCDIC stateful */
+  UFILE *myFile = u_fopen(STANDARD_TEST_FILE, "wb", STANDARD_TEST_LOCALE, enc);
+  FILE *myCFile;
+  int shift = 0;
+  int32_t i;
+
+  if (myFile == NULL) {
+    log_err("Can't write test file %s\n", STANDARD_TEST_FILE);
+    return;
+  }
+  
+  u_fprintf(myFile, "%S", utf16String);
+  u_fclose(myFile);
+
+  /* now read it back */
+    myCFile = fopen(STANDARD_TEST_FILE, "rb");
+    if (myCFile == NULL) {
+        log_err("Can't read test file.");
+        return;
+    }
+
+    inLen = fread(inBuf, 1, 200, myCFile);
+    fclose(myCFile);
+    
+    if(inLen<=0) {
+      log_err("Failed during read of test file.");
+      return;
+    }
+
+    /* check if shift in and out */
+    for(i=0;i<(int32_t)inLen;i++) {
+      if(inBuf[i]==0x0E) {  /* SO */
+        shift= 1;
+      } else if(inBuf[i]==0x0F) { /* SI */
+        shift= -1;
+      }
+    }
+    
+    if(shift==0) {
+      log_err("Err: shift was unchanged\n");
+    } else if(shift==1) {
+      log_err("Err: at end of string, we were still shifted out (SO, 0x0E).\n");
+    } else if(shift==-1) {
+      log_verbose("OK: Shifted in (SI, 0x0F)\n");
+    }
+
+    if(inLen != 12) {
+      log_err("Expected 12 bytes, read %d\n", inLen);
+    } else {
+      log_verbose("OK: read %d bytes\n", inLen);
+    }
+
+
+#endif
+}
+
 #if !UCONFIG_NO_FORMATTING
 static void TestFilePrintCompatibility(void) {
-    UFILE *myFile = u_fopen(STANDARD_TEST_FILE, "wb", "en_US_POSIX", NULL);
+    UFILE *myFile = u_fopen(STANDARD_TEST_FILE, "wb", STANDARD_TEST_LOCALE, NULL);
     FILE *myCFile;
     int32_t num;
     char cVal;
     static const UChar emptyStr[] = {0};
     char readBuf[512] = "";
     char testBuf[512] = "";
+    int32_t n = 0;
 
     if (myFile == NULL) {
         log_err("Can't write test file.\n");
         return;
     }
 #if !UCONFIG_NO_FORMATTING
-    if (strcmp(u_fgetlocale(myFile), "en_US_POSIX") != 0) {
+    if (strcmp(u_fgetlocale(myFile), STANDARD_TEST_LOCALE) != 0) {
         log_err("Got %s instead of en_US_POSIX for locale\n", u_fgetlocale(myFile));
     }
 #endif
@@ -958,38 +1028,39 @@ static void TestFilePrintCompatibility(void) {
     }
 
     for (num = -STANDARD_TEST_NUM_RANGE; num < STANDARD_TEST_NUM_RANGE; num++) {
-        fscanf(myCFile, "%s", readBuf);
+        /* Note: gcc on Ubuntu complains if return value of scanf is ignored. */
+        n += fscanf(myCFile, "%s", readBuf);
         sprintf(testBuf, "%x", (int)num);
         if (strcmp(readBuf, testBuf) != 0) {
             log_err("%%x Got: \"%s\", Expected: \"%s\"\n", readBuf, testBuf);
         }
 
-        fscanf(myCFile, "%s", readBuf);
+        n += fscanf(myCFile, "%s", readBuf);
         sprintf(testBuf, "%X", (int)num);
         if (strcmp(readBuf, testBuf) != 0) {
             log_err("%%X Got: \"%s\", Expected: \"%s\"\n", readBuf, testBuf);
         }
 
-        fscanf(myCFile, "%s", readBuf);
+        n += fscanf(myCFile, "%s", readBuf);
         sprintf(testBuf, "%o", (int)num);
         if (strcmp(readBuf, testBuf) != 0) {
             log_err("%%o Got: \"%s\", Expected: \"%s\"\n", readBuf, testBuf);
         }
 
         /* fprintf is not compatible on all platforms e.g. the iSeries */
-        fscanf(myCFile, "%s", readBuf);
+        n += fscanf(myCFile, "%s", readBuf);
         sprintf(testBuf, "%d", (int)num);
         if (strcmp(readBuf, testBuf) != 0) {
             log_err("%%d Got: \"%s\", Expected: \"%s\"\n", readBuf, testBuf);
         }
 
-        fscanf(myCFile, "%s", readBuf);
+        n += fscanf(myCFile, "%s", readBuf);
         sprintf(testBuf, "%i", (int)num);
         if (strcmp(readBuf, testBuf) != 0) {
             log_err("%%i Got: \"%s\", Expected: \"%s\"\n", readBuf, testBuf);
         }
 
-        fscanf(myCFile, "%s", readBuf);
+        n += fscanf(myCFile, "%s", readBuf);
         sprintf(testBuf, "%f", (double)num);
         if (strcmp(readBuf, testBuf) != 0) {
             log_err("%%f Got: \"%s\", Expected: \"%s\"\n", readBuf, testBuf);
@@ -1007,13 +1078,13 @@ static void TestFilePrintCompatibility(void) {
             log_err("%%E Got: \"%s\", Expected: \"%s\"\n", readBuf, testBuf);
         }*/
 
-        fscanf(myCFile, "%s", readBuf);
+        n += fscanf(myCFile, "%s", readBuf);
         sprintf(testBuf, "%g", (double)num);
         if (strcmp(readBuf, testBuf) != 0) {
             log_err("%%g Got: \"%s\", Expected: \"%s\"\n", readBuf, testBuf);
         }
 
-        fscanf(myCFile, "%s", readBuf);
+        n += fscanf(myCFile, "%s", readBuf);
         sprintf(testBuf, "%G", (double)num);
         if (strcmp(readBuf, testBuf) != 0) {
             log_err("%%G Got: \"%s\", Expected: \"%s\"\n", readBuf, testBuf);
@@ -1022,13 +1093,13 @@ static void TestFilePrintCompatibility(void) {
 
     /* Properly eat the newlines */
     for (num = 0; num < (int32_t)strlen(C_NEW_LINE); num++) {
-        fscanf(myCFile, "%c", &cVal);
+        n += fscanf(myCFile, "%c", &cVal);
         if (cVal != C_NEW_LINE[num]) {
             log_err("OS newline error\n");
         }
     }
     for (num = 0; num < (int32_t)strlen(C_NEW_LINE); num++) {
-        fscanf(myCFile, "%c", &cVal);
+        n += fscanf(myCFile, "%c", &cVal);
         if (cVal != C_NEW_LINE[num]) {
             log_err("ustdio newline error\n");
         }
@@ -1036,17 +1107,18 @@ static void TestFilePrintCompatibility(void) {
 
     for (num = 0; num < 0x80; num++) {
         cVal = -1;
-        fscanf(myCFile, "%c", &cVal);
+        n += fscanf(myCFile, "%c", &cVal);
         if (num != cVal) {
             log_err("%%c Got: 0x%x, Expected: 0x%x\n", cVal, num);
         }
     }
+    (void)n;
     fclose(myCFile);
 }
 #endif
 
 #define TestFPrintFormat(uFormat, uValue, cFormat, cValue) \
-    myFile = u_fopen(STANDARD_TEST_FILE, "w", "en_US_POSIX", NULL);\
+    myFile = u_fopen(STANDARD_TEST_FILE, "w", STANDARD_TEST_LOCALE, NULL);\
     if (myFile == NULL) {\
         log_err("Can't write test file for %s.\n", uFormat);\
         return;\
@@ -1057,7 +1129,7 @@ static void TestFilePrintCompatibility(void) {
     \
     uNumPrinted = u_fprintf(myFile, uFormat, uValue);\
     u_fclose(myFile);\
-    myFile = u_fopen(STANDARD_TEST_FILE, "r", "en_US_POSIX", NULL);\
+    myFile = u_fopen(STANDARD_TEST_FILE, "r", STANDARD_TEST_LOCALE, NULL);\
     u_fgets(uBuffer, sizeof(uBuffer)/sizeof(*uBuffer), myFile);\
     u_fclose(myFile);\
     u_austrncpy(compBuffer, uBuffer, sizeof(uBuffer)/sizeof(*uBuffer));\
@@ -1188,14 +1260,14 @@ static void TestFprintfFormat(void) {
     TestFPrintFormat("%3f", 1.234,       "%3f", 1.234);
     TestFPrintFormat("%3f", -1.234,      "%3f", -1.234);
 
-    myFile = u_fopen(STANDARD_TEST_FILE, "w", "en_US_POSIX", NULL);
+    myFile = u_fopen(STANDARD_TEST_FILE, "w", STANDARD_TEST_LOCALE, NULL);
     /* Reinitialize the buffer to verify null termination works. */
     u_memset(uBuffer, 0x2a, sizeof(uBuffer)/sizeof(*uBuffer));
     memset(buffer, '*', sizeof(buffer)/sizeof(*buffer));
     
     uNumPrinted = u_fprintf(myFile, "%d % d %d", -1234, 1234, 1234);
     u_fclose(myFile);
-    myFile = u_fopen(STANDARD_TEST_FILE, "r", "en_US_POSIX", NULL);
+    myFile = u_fopen(STANDARD_TEST_FILE, "r", STANDARD_TEST_LOCALE, NULL);
     u_fgets(uBuffer, sizeof(uBuffer)/sizeof(*uBuffer), myFile);
     u_fclose(myFile);
     u_austrncpy(compBuffer, uBuffer, sizeof(uBuffer)/sizeof(*uBuffer));
@@ -1236,7 +1308,7 @@ static void TestFScanSetFormat(const char *format, const UChar *uValue, const ch
     
     u_fprintf(myFile, "%S", uValue);
     u_fclose(myFile);
-    myFile = u_fopen(STANDARD_TEST_FILE, "r", "en_US_POSIX", NULL);
+    myFile = u_fopen(STANDARD_TEST_FILE, "r", STANDARD_TEST_LOCALE, NULL);
     uNumScanned = u_fscanf(myFile, format, uBuffer);
     u_fclose(myFile);
     if (expectedToPass) {
@@ -1321,7 +1393,7 @@ static void TestBadFScanfFormat(const char *format, const UChar *uValue, const c
     
     u_fprintf(myFile, "%S", uValue);
     u_fclose(myFile);
-    myFile = u_fopen(STANDARD_TEST_FILE, "r", "en_US_POSIX", NULL);
+    myFile = u_fopen(STANDARD_TEST_FILE, "r", STANDARD_TEST_LOCALE, NULL);
     uNumScanned = u_fscanf(myFile, format, uBuffer);
     u_fclose(myFile);
     if (uNumScanned != 0 || uBuffer[0] != 0x2a || uBuffer[1] != 0x2a) {
@@ -1345,7 +1417,7 @@ static void Test_u_vfprintf(const char *expectedResult, const char *format, ...)
     int32_t count;
     UFILE *myFile;
 
-    myFile = u_fopen(STANDARD_TEST_FILE, "w", "en_US_POSIX", "UTF-8");
+    myFile = u_fopen(STANDARD_TEST_FILE, "w", STANDARD_TEST_LOCALE, "UTF-8");
     if (!myFile) {
         log_err("Test file can't be opened\n");
         return;
@@ -1353,12 +1425,13 @@ static void Test_u_vfprintf(const char *expectedResult, const char *format, ...)
 
     va_start(ap, format);
     count = u_vfprintf(myFile, format, ap);
+    (void)count;    /* Suppress set but not used warning.  */
     va_end(ap);
 
     u_fclose(myFile);
 
 
-    myFile = u_fopen(STANDARD_TEST_FILE, "r", "en_US_POSIX", "UTF-8");
+    myFile = u_fopen(STANDARD_TEST_FILE, "r", STANDARD_TEST_LOCALE, "UTF-8");
     if (!myFile) {
         log_err("Test file can't be opened\n");
         return;
@@ -1371,7 +1444,7 @@ static void Test_u_vfprintf(const char *expectedResult, const char *format, ...)
     u_fclose(myFile);
 
 
-    myFile = u_fopen(STANDARD_TEST_FILE, "w", "en_US_POSIX", NULL);
+    myFile = u_fopen(STANDARD_TEST_FILE, "w", STANDARD_TEST_LOCALE, NULL);
     if (!myFile) {
         log_err("Test file can't be opened\n");
         return;
@@ -1385,7 +1458,7 @@ static void Test_u_vfprintf(const char *expectedResult, const char *format, ...)
     u_fclose(myFile);
 
 
-    myFile = u_fopen(STANDARD_TEST_FILE, "r", "en_US_POSIX", NULL);
+    myFile = u_fopen(STANDARD_TEST_FILE, "r", STANDARD_TEST_LOCALE, NULL);
     if (!myFile) {
         log_err("Test file can't be opened\n");
         return;
@@ -1513,6 +1586,8 @@ U_CFUNC void
 addFileTest(TestNode** root) {
 #if !UCONFIG_NO_FORMATTING
     addTest(root, &TestFile, "file/TestFile");
+    addTest(root, &TestFinit, "file/TestFinit");
+    addTest(root, &TestFadopt, "file/TestFadopt");
 #endif
     addTest(root, &StdinBuffering, "file/StdinBuffering");
     addTest(root, &TestfgetsBuffers, "file/TestfgetsBuffers");
@@ -1522,6 +1597,7 @@ addFileTest(TestNode** root) {
     addTest(root, &TestfgetsNewLineCount, "file/TestfgetsNewLineCount");
     addTest(root, &TestFgetsLineBuffering, "file/TestFgetsLineBuffering");
     addTest(root, &TestCodepage, "file/TestCodepage");
+    addTest(root, &TestCodepageFlush, "file/TestCodepageFlush");
     addTest(root, &TestFileWriteRetvalUTF16, "file/TestFileWriteRetvalUTF16");
     addTest(root, &TestFileWriteRetvalUTF8, "file/TestFileWriteRetvalUTF8");
     addTest(root, &TestFileWriteRetvalASCII, "file/TestFileWriteRetvalASCII");
