@@ -1,10 +1,12 @@
 /*
  *
- * (C) Copyright IBM Corp. 1998-2007 - All Rights Reserved
+ * (C) Copyright IBM Corp. 1998-2014 - All Rights Reserved
  *
  */
 
-#include "LETypes.h"
+#ifndef USING_ICULEHB /* C API not available under HB */
+
+#include "layout/LETypes.h"
 #include "loengine.h"
 #include "PortableFontInstance.h"
 #include "SimpleFontInstance.h"
@@ -38,6 +40,13 @@ const char *le_getNameString(le_font *font, le_uint16 nameID, le_uint16 platform
 	return pfi->getNameString(nameID, platform, encoding, language);
 }
 
+const LEUnicode16 *le_getUnicodeNameString(le_font *font, le_uint16 nameID, le_uint16 platform, le_uint16 encoding, le_uint16 language)
+{
+	PortableFontInstance *pfi = (PortableFontInstance *) font;
+
+	return pfi->getUnicodeNameString(nameID, platform, encoding, language);
+}
+
 void le_deleteNameString(le_font *font, const char *name)
 {
 	PortableFontInstance *pfi = (PortableFontInstance *) font;
@@ -45,6 +54,13 @@ void le_deleteNameString(le_font *font, const char *name)
 	pfi->deleteNameString(name);
 }
 
+void le_deleteUnicodeNameString(le_font *font, const LEUnicode16 *name)
+{
+	PortableFontInstance *pfi = (PortableFontInstance *) font;
+
+	pfi->deleteNameString(name);
+}
+
 le_uint32 le_getFontChecksum(le_font *font)
 {
 	PortableFontInstance *pfi = (PortableFontInstance *) font;
@@ -53,3 +69,4 @@ le_uint32 le_getFontChecksum(le_font *font)
 }
 
 U_CDECL_END
+#endif
