@@ -1,7 +1,7 @@
 /*
  *******************************************************************************
  *
- *   Copyright (C) 1999-2006, International Business Machines
+ *   Copyright (C) 1999-2014, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  *
  *******************************************************************************
@@ -14,11 +14,19 @@
 #ifndef __LETEST_H
 #define __LETEST_H
 
+
+#ifdef USING_ICULEHB
+#include "layout/LETypes.h"
+#else
 #include "LETypes.h"
+#endif
+#include "unicode/ctest.h"
 
 #include <stdlib.h>
 #include <string.h>
 
+U_NAMESPACE_USE
+
 #define ARRAY_SIZE(array) (sizeof array / sizeof array[0])
 
 #define ARRAY_COPY(dst, src, count) memcpy((void *) (dst), (void *) (src), (count) * sizeof (src)[0])
@@ -36,4 +44,11 @@ struct TestResult
     le_int32  *indices;
     float     *positions;
 };
+
+#ifndef __cplusplus
+typedef struct TestResult TestResult;
+#endif
+
+U_CFUNC void addCTests(TestNode **root);
+
 #endif
