@@ -1,15 +1,12 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2002-2004, International Business Machines
+*   Copyright (C) 2002-2014, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
 *
 * File genrb.h
-*
-* Modification History:
-*******************************************************************************
 */
 
 #ifndef GENRB_H
@@ -24,9 +21,9 @@
 
 
 #include "ucbuf.h"
-#include "error.h"
+#include "errmsg.h"
 #include "parse.h"
-#include "util.h"
+#include "rbutil.h"
 #include "reslist.h"
 
 
@@ -37,7 +34,19 @@
 #include "unicode/uloc.h"
 
 /* The version of genrb */
-#define GENRB_VERSION "3.2"
+#define GENRB_VERSION "3.3"
+
+U_CDECL_BEGIN
+
+U_CAPI void processFile(
+    const char *filename,
+    const char* cp,
+    const char *inputDir,
+    const char *outputDir,
+    const char *packageName,
+    UBool omitBinaryCollation,
+    UErrorCode *status);
+
+U_CDECL_END
 
 #endif
-
